#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>

#include "trxproj.h"       /* particle types */
#include "trxxs.h"         /* interaction types */
#include "trxparticle.h"   /* event structure */

#include "trxeval.h"       /* eval structure */

/*static*/ char rcsid[] = {"$Id: trxevalusr.c,v 1.2 2008/06/20 14:39:40 kraemer Exp $"};

/** \file
   Template for an external user evaluation function.
*/
/*

   Copyright  (C) 2008-2010 M.Kraemer, GSI, 
                             all rights reserved
   $Log: trxevalusr.c,v $
   Revision 1.2  2008/06/20 14:39:40  kraemer
   x98-libs, user function, tilted vol's

   Revision 1.1  2008/02/05 11:03:46  kraemer
   user plug-in


*/

/* --- begin user specific (declarations) */
                                      /** \struct SEVALUSR
                                          \brief Local user evaluation.
                                      */ 
static struct SEVALUSR {
   int nE, nA;
   unsigned int uiEScale; 
   double *pdEBin, *pdABin;
   unsigned int **puiSpcAE;          /* [nA][nE] spectrum */
   double dvLimPhi[2];
} sUsr;

/* --- end user specific (declarations) */

/** User evaluation function.
*/

int TRXEvalUsr  (
                  struct STRXEVAL *psEval,      /**< upd: evaluation info */
                  struct STRXPARTICLEEVT *psPE, /**< inp: [] of events */
                  int nPE,                      /**< inp: # of events */
                  unsigned int uiAction,        /**< inp: evaluation action code */
                  unsigned int uiFlag           /**< inp: function action flag */
                ) {

   char *pcDbgMsg = __FILE__; char *pcMsg = (char *) pcDbgMsg;
   struct SCMDERR sCE;
   unsigned int uiFlagCE = 0;

   struct SCMDPRTCL *psCP = NULL;

/* --- begin user specific (declarations) */

#define BACKSCATTER_THRESHOLD 50E-3
#define ME 32    
#define MA 1
   int nE = ME, nA = MA;
   size_t nv[] = {0,0};
   int nDim=0;
   ssize_t lE, lA; 
   unsigned int uiEScale=TRXEVAL_LIN; 
   unsigned int uiSumAll, uiSumThr, uiBacksc=0, uiBackscThr=0, uiTransm=0;
   
   double dvLimE[] = { 0, 1E0 };
   double dvLimA[] = { 0, 12E0 };
   double dvLimPhi[] = { -180, 180 };
   double dFactor=0, dErr=0, dd=0, dEE=0, ddE=0;
   double dThFactor;
   FILE *fp = NULL;
   int iPE;
   char cStr[128] = {""};

/* --- end user specific (declarations) */


if ( uiFlag & TRXEVAL_DEBUG ) fprintf( stderr, "<D> %s.....\n", pcDbgMsg );
   if ( uiFlag & TRXEVAL_DEBUG ) uiFlagCE  |= CMDERR_DEBUG;
   CMDERRINIT( sCE );
                                      /* check some input */
   sCE.iRC = ERRTRXEVAL_BADADDR;
   if ( psEval == NULL ) goto gError;
                                      /* locate protocol */
   if ( psEval -> psCmd != NULL ) psCP = &(psEval->psCmd->sCP);
                                      /* dispatch actions */
   switch(uiAction) {
                                      /* initial setup */
   case TRXEVAL_SETUP:

/* --- begin user specific (setup) */
                                      /* prepare histograms */
if ( uiFlag & TRXEVAL_DEBUG ) fprintf( stderr, "<D> setup...\n" );
                                      /* reset user evaluation */
      memset( &sUsr, 0, sizeof(sUsr) );
                                      /* evaluate user string, if any */
      if (  psEval->pcUsrStr != NULL ) 
      if ( *psEval->pcUsrStr != 0 )
         sscanf( psEval->pcUsrStr, 
	                "%d %lG %lG %d %lG %lG %lG %lG", 
		               &nE, &dvLimE[0], &dvLimE[1], 
		               &nA, &dvLimA[0], &dvLimA[1],
		               &dvLimPhi[0], &dvLimPhi[1] 
	        );
                                      /* allocate energy histogram */
      if ( ( sUsr.pdEBin    = calloc( nE+1, sizeof(*(sUsr.pdEBin))) ) == NULL ) {
         sCE.iRC = ERRTRXEVAL_SYS; goto gError;
      }   
      sUsr.nE = nE;
                                      /* lin/log energy scale */
      if ( uiEScale == TRXEVAL_LOG ) {
         dvLimE[0] = 1E-3;
         vrampd( log(dvLimE[0]), log(dvLimE[1]), sUsr.pdEBin, 1, nE+1 );
         vexp ( sUsr.pdEBin, 1,      sUsr.pdEBin, 1, nE+1 );
      } else {
         vrampd( dvLimE[0], dvLimE[1], sUsr.pdEBin, 1, nE+1 );
      }
      if ( dvLimE[1] <= dvLimE[0] ) {
         sCE.iRC = ERRTRXEVAL_BADLIME; goto gError;
      }        
                                      /* allocate polar angle histogram */
      if ( ( sUsr.pdABin    = calloc( nA+1, sizeof(*(sUsr.pdABin))) ) == NULL ) {
         sCE.iRC = ERRTRXEVAL_SYS; goto gError;
      }   
      sUsr.nA = nA;
                                      /*  */
      vrampd( dvLimA[1], dvLimA[0], sUsr.pdABin, 1, nA+1 );
      vcosd ( sUsr.pdABin, 1, sUsr.pdABin, 1, nA+1 );
                                      /* record phi limits */
      sUsr.dvLimPhi[0] = dvLimPhi[0];
      sUsr.dvLimPhi[1] = dvLimPhi[1];
                                      /* allocate 2-dim histogram */
      nv[0] = nA; nv[1] = nE;
      sUsr.puiSpcAE = dimalloc( sizeof(**(sUsr.puiSpcAE)), nDim=2, nv );
      if ( sUsr.puiSpcAE == NULL ) {
         sCE.iRC = ERRTRXEVAL_SYS; goto gError;
      }  
                                      /* remember "private" data */
      psEval -> pvEval	= &sUsr;	 		      
if ( uiFlag & TRXEVAL_DEBUG ) fprintf( stderr, "<D> setup OK\n" );

/* --- end user specific (setup) */

   break;

                                      /* dispatch events (called for a block of events) */
   case TRXEVAL_EVENT:

/* --- begin user specific */

if ( uiFlag & TRXEVAL_DEBUG ) fprintf( stderr, "<D> event...\n" );
                                      /* for security, check some ptr.s */
      sCE.iRC = ERRTRXEVAL_BADADDR;
      if ( psPE == NULL ) goto gError;
      if ( sUsr.pdEBin == NULL ) goto gError;
      if ( sUsr.pdABin == NULL ) goto gError;
      if ( sUsr.puiSpcAE == NULL ) goto gError;

/* --- end user specific */
                                      /* loop over block of events */
      for ( iPE=0; iPE<nPE; iPE++ ) { 

                                      /*  interaction type */
         switch(psPE->i32IA) {
                                      /* begin-of-run */
	        case TRXXS_BOR:
	           psEval -> uiEvt = psPE -> ui32Evt;
	        break;
                                      /* end-of-run, record total # of primaries  */
	        case TRXXS_EOR:
	           psEval -> uiEvt = psPE -> ui32Evt;
	        break;
                                      /* everything else */
	        default:

/* --- begin user specific */

            if ( psPE->ui32IASequence > 0 ) 
            if ( psPE->rE > 15E-3 ) 
            if ( psPE->i32IA == TRXXS_OUTSIDE ) {
            if ( psPE->ui32PT == TRXPROJ_ELECTRON ) {
#ifndef ATAN2D
#define ATAN2D(y,x) ( atan2( (y),(x) ) * 180.0E0 / PI )
#endif
                                      /* sort energy, polar angle, azimuth angle */
               dd = psPE -> rE;
               vbind( &dd, 1, 1, sUsr.pdEBin, sUsr.nE+1, &lE, 1 ); 
               dd = psPE -> rvd[2];
               vbind( &dd, 1, 1, sUsr.pdABin, sUsr.nA+1, &lA, 1 );
               if ( ( lE >= 0 ) && ( lE < sUsr.nE ) &&
                    ( lA >= 0 ) && ( lA < sUsr.nA )
                  ) {
                 dd = 0;
                 if ( ( psPE -> rvd[1] != 0 ) || ( psPE -> rvd[0] != 0 ) )
		                  dd = ATAN2D( psPE -> rvd[1], psPE -> rvd[0] );
		               if ( (dd>=-180) && (dd<=180) )
                    sUsr.puiSpcAE[lA][lE] ++;
               }
            }
            }

/* --- end user specific */
                           
         break;
	        } /* interaction */ 
         psPE++;
      } /* event loop */
   break;      

                                      /* end-of-file */
   case TRXEVAL_EOF:

/* --- begin user specific */
                                      /* output results */
if ( uiFlag & TRXEVAL_DEBUG ) fprintf( stderr, "<D> EOF...\n" );
                                      /* for security some checks */
      sCE.iRC = ERRTRXEVAL_BADADDR;
      if ( psEval->pcOutFile == NULL ) goto gError;
      sCE.iRC = ERRTRXEVAL_BADFILENAME;
      if ( *(psEval->pcOutFile) == '\0' ) goto gError;
      sCE.iRC = ERRTRXEVAL_BADNEVT;
      if ( psEval -> uiEvt <= 0 ) goto gError;
      
                                      /* open outfile */
      if ( ( fp = fopen( psEval->pcOutFile, "w" ) ) == NULL ) {
         sCE.iRC = ERRTRXEVAL_SYS; goto gError;
      }
      
                                      /* output results */
      fprintf( fp, "Outside\n" );
      fprintf( fp, "c: # events=%lu\n", psEval->uiEvt );
      fprintf( fp, "x: Ebl^keVbr^\n" );
      fprintf( fp, "y: EdN/dE/dOm^ bl^1/srbr^\n" );
      fprintf( fp, "p: nosp ylog xlin\n" );

      for ( lA=0; lA<sUsr.nA; lA++ ) {
         fprintf( fp, "c: %.3G0@ < th^ < %.3G0@\n",
                       acos( sUsr.pdABin[lA  ] ) * 180E0 / PI,
                       acos( sUsr.pdABin[lA+1] ) * 180E0 / PI );
         fprintf( fp, "H: x y(%.3G0@<th^<%.3G0@),hlt0 d%% n d%%\n",
                       acos( sUsr.pdABin[lA  ] ) * 180E0 / PI,
                       acos( sUsr.pdABin[lA+1] ) * 180E0 / PI );
         dFactor = fabs( sUsr.pdABin[lA+1] - sUsr.pdABin[lA] ) * 
	           (double) psEval->uiEvt * (PI+PI);

         uiSumAll = uiSumThr = 0;
         for ( lE=0; lE<sUsr.nE; lE++ ) {
            dErr = 0E0;
            if ( sUsr.puiSpcAE[lA][lE] > 0 )
               dErr = 1E2 / sqrt( (double) sUsr.puiSpcAE[lA][lE] );
            uiSumAll += sUsr.puiSpcAE[lA][lE];
            if ( sUsr.pdEBin[lE] > BACKSCATTER_THRESHOLD )
               uiSumThr += sUsr.puiSpcAE[lA][lE];
            dEE = ( sUsr.pdEBin[lE+1] + sUsr.pdEBin[lE] ) * 0.5;
            ddE = ( sUsr.pdEBin[lE+1] - sUsr.pdEBin[lE] ) ;
            dd  = ( (double) sUsr.puiSpcAE[lA][lE] ) / ( dFactor * ddE );
            fprintf(fp,"% E % E % E % E % E\n", sUsr.pdEBin[lE],
                          dd * dEE, dErr, dd, dErr );               
         }   
         fprintf(fp,"% E % E % E % E % E\n", sUsr.pdEBin[lE],
                       dd * dEE * 1E-6, dErr * 1E-6, dd * 1E-6, dErr * 1E-6 );               
         if ( sUsr.pdABin[lA] < 0E0 ) { uiBacksc += uiSumAll; uiBackscThr += uiSumThr; }
         if ( sUsr.pdABin[lA] > 0E0 ) { uiTransm += uiSumAll; }
      }  
                                      /* angular distrib. */    
      fprintf( fp, "n: x y(ang. distr.),hlt0 d\n" );
      uiSumAll = 0;
#ifndef ACOSD
#define ACOSD(x) ( acos( x ) * 180.0E0 / PI )
#endif
      for ( lA=sUsr.nA-1; lA>=0; lA-- ) {
         uiSumAll = 0;
         dFactor = ( sUsr.pdABin[lA+1] - sUsr.pdABin[lA] ) * 
	          (double) psEval->uiEvt * (PI+PI);
         dThFactor = fabs( ACOSD(sUsr.pdABin[lA+1]) - ACOSD(sUsr.pdABin[lA]) ) * 
	             (double) psEval->uiEvt * (PI+PI);
         for ( lE=0; lE<sUsr.nE; lE++) {
            uiSumAll += sUsr.puiSpcAE[lA][lE];
         }
         fprintf( fp, "%G %G %G %G %G\n",
                      acos( sUsr.pdABin[lA+1] ) * 180E0 / PI,
                      (double) uiSumAll / dFactor,
                sqrt( (double) uiSumAll ) / dFactor,
                      (double) uiSumAll / dThFactor,
                sqrt( (double) uiSumAll ) / dThFactor
                );
      }
      fprintf( fp, "%G %G %G %G %G\n",
                   acos( sUsr.pdABin[lA+1] ) * 180E0 / PI,
                   (double) uiSumAll / dFactor * 1E-6, 
             sqrt( (double) uiSumAll ) / dFactor * 1E-6,
                   (double) uiSumAll / dThFactor * 1E-6, 
             sqrt( (double) uiSumAll ) / dThFactor * 1E-6
             );
                                      /* energy  distrib. */
      fprintf( fp, "n: x y(energy distr.),hlt0 d\n" );
      uiSumAll = 0;
      for ( lE=0; lE<sUsr.nE; lE++) {
         uiSumAll = 0;
         ddE = ( sUsr.pdEBin[lE+1] - sUsr.pdEBin[lE] ) * (double) psEval->uiEvt;
         for ( lA=0; lA<sUsr.nA; lA++ ) {
            uiSumAll += sUsr.puiSpcAE[lA][lE];
         }
         fprintf( fp, "%G %G %G\n", 
                      sUsr.pdEBin[lE], 
                            (double) uiSumAll / ddE, 
                      sqrt( (double) uiSumAll ) / ddE );
      }
      fprintf( fp, "%G %G %G\n", 
                   sUsr.pdEBin[lE], 
                         (double) uiSumAll / ddE * 1E-6, 
                   0E0 );

                                      /* */
      fprintf( fp, "n2: (y=" );
      for ( lA=0; lA<sUsr.nA; lA++) {
         fprintf( fp, " %G", ( sUsr.pdABin[lA+1] + sUsr.pdABin[lA] ) * 0.5 );
      }
      fprintf( fp, ") (x=" );
      for ( lE=0; lE<sUsr.nE; lE++) {
         fprintf( fp, " %G", ( sUsr.pdEBin[lE+1] + sUsr.pdEBin[lE] ) * 0.5 );
      }
      fprintf( fp, ") z\n" );
      for ( lA=sUsr.nA-1; lA>=0; lA-- ) {
         for ( lE=0; lE<sUsr.nE; lE++) {
            fprintf( fp, " %G", (double) sUsr.puiSpcAE[lA][lE] );
         }
         fprintf( fp, "\n" );
      }
                                      /* */
      if ( psEval -> uiEvt > 0 ) {
         sprintf( cStr, "<I> Backscatter  coefficient > 0  eV = %G +- %G",
                          (double) uiBacksc   / (double) psEval -> uiEvt,
                    sqrt( (double) uiBacksc ) / (double) psEval -> uiEvt
                );
         _cmdprtcl( psCP, cStr, CMDPRTCL_FILE | CMDPRTCL_TERM );       
         sprintf( cStr, "<I> Backscatter  coefficient > %G eV = %G +- %G",
                          BACKSCATTER_THRESHOLD * 1E3,
                          (double) uiBackscThr   / (double) psEval -> uiEvt,
                    sqrt( (double) uiBackscThr ) / (double) psEval -> uiEvt
                );
         _cmdprtcl( psCP, cStr, CMDPRTCL_FILE | CMDPRTCL_TERM | CMDPRTCL_NOTIME );       
         sprintf( cStr, "<I> Transmission coefficient > 0  eV = %G +- %G",
                          (double) uiTransm   / (double) psEval -> uiEvt,
                    sqrt( (double) uiTransm ) / (double) psEval -> uiEvt
                );
         _cmdprtcl( psCP, cStr, CMDPRTCL_FILE | CMDPRTCL_TERM | CMDPRTCL_NOTIME );       
      }
                                      /* */
      if ( fp ) fclose(fp); fp = NULL;

/* --- end user specific */

   break;
                                      /* end of evaluation, cleanup */
   case TRXEVAL_CLEANUP:

/* --- begin user specific */

if ( uiFlag & TRXEVAL_DEBUG ) fprintf( stderr, "<D> cleanup...\n" );
      if ( sUsr.pdEBin ) free( sUsr.pdEBin ); sUsr.pdEBin = NULL; 
      if ( sUsr.pdABin ) free( sUsr.pdABin ); sUsr.pdABin = NULL; 
      nv[0] = sUsr.nA; 
      nv[1] = sUsr.nE; 
      if ( sUsr.puiSpcAE ) dimfree( (void **) sUsr.puiSpcAE, nDim=2, nv ); sUsr.puiSpcAE = NULL; 
      memset( &sUsr, 0, sizeof(sUsr) );

/* --- end user specific */

      psEval -> pvEval = NULL;
   break;
 
   default:
if ( uiFlag & TRXEVAL_DEBUG ) fprintf( stderr, "<D> default...\n" );
   break;   

   } 
                                      /*  */
   sCE.iRC = sCE.iSev = 0;
gError:;
   if ( psEval ) psEval -> sCE = sCE;
   if ( sCE.iSev ) {
      sCE.pcMsg = pcMsg;
      TRXEvalError( &sCE, uiFlagCE );
   }
   
if ( uiFlag & TRXEVAL_DEBUG ) fprintf( stderr, "<D> %s done\n", pcDbgMsg );
   return(sCE.iSev);
}
