#ifndef DCMBASIC_H
#define DCMBASIC_H
/** \file

   Definitions to handle DICOM files.

*/
/*

   Copyright  2005-2011 M.Kraemer, GSI, D-64291 Darmstadt

   $Log$

*/
#define UMEMCPY( pto, pfr ) memcpy( pto, pfr, sizeof(*(pto)) ); 
#define UMEMCLR( pto      ) memset( pto, 0, sizeof(*(pto)) ); 
#define UMEMALLOC( pto, nn ) ( pto = calloc( nn, sizeof(*(pto)) ) )
                                    
#include "ummap.h"
#include "cmderr.h"

                                      /* global flags */
#define DICOM_MASKGLOBAL 0xFF000000/* global (e.g. debug) */
#define DICOM_MASKFUNC   0x00FF0000/* function specific flags */
#define DICOM_MASKQUAL   0x0000FFFF/* for additional qualifiers */

#define DICOM_DEBUG  0x80000000    /* general check flag */
#define DICOM_QUIET  0x10000000    /* suppress error messages */
#define DICOM_NOMSG  DICOM_QUIET

/*
    cmddicom    
      cmddcm
       dicom
         dcm
C.2.1       PATREL    Patient Relationship Module
C.2.2       PATIDF    Patient Identification Module
C.2.3       PATDEM    Patient Demographic Module
C.2.4       PATMED    Patient Medical Module
C.3.1       VISREL    Visit Relationship Module

C.3.2       VISIDF    Visit Identification Module
C.3.3       VISSTS    Visit Status Module
C.3.4       VISADM    Visit Admission Module
C.3.5       VISDIS    Visit Discharge Module (Retired. See PS 3.3 2004.)
C.3.6       VISSCHD   Visit Scheduling Module (Retired. See PS 3.3 2004.)
C.4.1       STDREL    Study Relationship Module (Retired. See PS 3.3 2004.)
C.4.2       STDIDF    Study Identification Module (Retired. See PS 3.3 2004.)
C.4.3       STDCLS    Study Classification Module (Retired. See PS 3.3 2004.)
C.4.4       STDSCHD   Study Scheduling Module (Retired. See PS 3.3 2004.)
C.4.5       STDACQ    Study Acquisition Module (Retired. See PS 3.3 2004.)
C.4.6       STDREAD   Study Read Module (Retired. See PS 3.3 2004.)
C.4.7       STDCMP    Study Component Module (Retired. See PS 3.3 2004.)
C.4.8       STDCMPREL Study Component Relationship Module (Retired. See PS 3.3 2004.)
C.4.9       STDCMPACQ Study Component Acquisition Module (Retired. See PS 3.3 2004.)
C.4.10      PRCSSTEP  Scheduled Procedure Step Module (Retired. See PS 3.3 2004.)
C.4.11      PRCREQ    Requested Procedure Module
C.4.12      IMGSRVREQ Imaging Service Request Module
C.4.13      PRCPSTEPR Performed Procedure Step Relationship
C.4.14      PRCPSTEPI Performed Procedure Step Information
C.4.15      IMGACQRES Image Acquisition Results
C.4.16      RADDOS    Radiation Dose
C.4.17      BMMC      Billing and Material Management Codes
C.4.18      GPxxxx    General Purpose Scheduled Procedure Step Relationship Module
C.4.19      GPxxxx    General Purpose Scheduled Procedure Step Information Module
C.4.20      GPxxxx    General Purpose Performed Procedure Step Relationship Module
C.4.21      GPxxxx    General Purpose Performed Procedure Step Information Module
C.4.22      GPxxxx    General Purpose Results
C.4.23      ???       Instance Availability Notification Module
C.7.1.1     PAT       Patient Module
C.7.1.2     SMIDF     Specimen Identification Module

C.7.1.3     CTS       Clinical Trial Subject Module

C.7.2                 Common Study IE Modules
C.7.2.1     STDGEN    General Study Module
C.7.2.2     STDPAT    Patient Study Module
C.7.2.3     STDCT     Clinical Trial Study Module

C.7.3                 Common Series IE Modules
C.7.3.1     SERGEN    General Series Module
C.7.3.2     SERCT     Clinical Trial Series Module
C.7.3.3     SERENH    Enhanced Series Module

C.7.4.1     FOR       Frame Of Reference Module
C.7.4.2     SYNC      Synchronization Module

C.7.5                 Common Equipment IE Modules
C.7.5.1     EQPGEN    General Equipment Module
C.7.5.2     EQPGENH   Enhanced General Equipment Module

C.7.6                 Common Image IE Modules
C.7.6.1     IMGGEN    General Image Module
C.7.6.2     IMGPLN    Image Plane Module
C.7.6.3     IMGPIX    Image Pixel Module
C.7.6.4     CONBOL    Contrast/Bolus Module
C.7.6.4b    CONBOLH   Enhanced Contrast/Bolus Module
C.7.6.5     CINE      Cine Module
C.7.6.6     MF        Multi-Frame Module
C.7.6.7     BIPLNS    Bi-Plane Sequence Module (Retired) 
C.7.6.8     BIPLNI    Bi-Plane Image Module (Retired) 
C.7.6.9     FP        Frame Pointers Module
C.7.6.10    MASK      Mask Module
C.7.6.11    DSPSHUT   Display Shutter Module
C.7.6.12    DEVICE    Device Module
C.7.6.13    INTERV    Intervention Module
C.7.6.14    ACQCTX    Acquisition Context Module
C.7.6.15    DSPSHUTBM Bitmap Display Shutter Module
C.7.6.16    MFFCT     Multi-frame Functional Groups Module
C.7.6.17    MFDIM     Multi-frame Dimension Module
C.7.6.18.1  SYNCCARD  Cardiac Synchronization Module
C.7.6.18.3  SYNCBULK  Bulk Motion Synchronization Module
C.7.6.19    LUTSPC    Supplemental Palette Color Lookup Table Module
C.7.6.20    PATO      Patient Orientation Module
C.7.6.21    ??????    Image Equipment Coordinate Relationship Module

C.7.7       PATSUM    Patient Summary Module (Retired.See PS 3.3 2004.)
C.7.8       STD???    Study Content Module (Retired. See PS 3.3 2004.)
C.7.9       ??????    Palette Color Lookup Table Module

C.8.1.1     CRSER     CR Series Module
C.8.1.2     CRIMG     CR Image Module

C.8.2.1     CTIMG     CT Image Module

C.8.3.1     MRIMG     MR Image Module

C.8.4.1     NMSER     NM Series Module (Retired) 
C.8.4.2     NMEQP     NM Equipment Module (Retired) 
C.8.4.3     NMIMG     NM Image Module (Retired) 
C.8.4.4     NMSAI     NM SPECT Acquisition Image Module (Retired) 
C.8.4.5     NMMGAIMG  NM Multi-gated Acquisition Image Module (Retired)
C.8.4.6     NMPPO     NM/PET Patient Orientation Module
C.8.4.7     NMIMGPIX  NM Image Pixel Module
C.8.4.8     NMMF      NM Multi-frame Module
C.8.4.9     NMIMG     NM Image Module
C.8.4.10    NMISO     NM Isotope Module
C.8.4.11    NMDET     NM Detector Module
C.8.4.12    NMTOMA    NM TOMO Acquisition Module
C.8.4.13    NMMGA     NM Multi-gated Acquisition Module
C.8.4.14    NMPHS     NM Phase Module

C.8.5.1     USFOR     US Frame of Reference Module (Retired) 
C.8.5.2     USREGCAL  US Region Calibration (Retired) 
C.8.5.3     USIMG     US Image Module (Retired) 
C.8.5.4     USFORS    US Frame of Reference Module Section (Retired. See PS 3.3 2003.)
C.8.5.5     USREGCAL  US Region Calibration Module
C.8.5.6     USIMG     US Image Module

C.8.6.1     SCEQP     SC Equipment Module
C.8.6.2     SCIMG     SC Image Module
C.8.6.3     SCMFIMG   SC Multi-frame Image Module
C.8.6.4     SCMFVEC   SC Multi-frame Vector Module

C.8.7.1     XRIMG     X-Ray Image Module
C.8.7.2     XRACQ     X-Ray Acquisition Module
C.8.7.4     XRTBL     X-Ray Table Module
C.8.7.5     XAPOS     XA Positioner Module
C.8.7.6     XRFPOS    XRF Positioner Module
C.8.7.7     XRTOMACQ  X-Ray Tomography Acquisition Module
C.8.7.8     XRACQDOS  X-Ray Acquisition Dose Module
C.8.7.9     XRGEN     X-Ray Generation Module
C.8.7.10    XRFLT     X-Ray Filtration Module
C.8.7.11    XRGRD     X-Ray Grid Module

C.8.8.1     RTSER     RT Series Module
C.8.8.2     RTIMG     RT Image Module
C.8.8.3     RTDOS     RT Dose Module
C.8.8.4     RTDVH     RT DVH Module
C.8.8.5     RTSS      Structure Set Module
C.8.8.6     RTROICNT  ROI Contour Module
C.8.8.7     RTDOSROI  RT Dose ROI Module
C.8.8.8     RTROIOBS  RT ROI Observations Module
C.8.8.9     RTPLN     RT General Plan Module
C.8.8.10    RTPSCRPT  RT Prescription Module
C.8.8.11    RTTOLTBL  RT Tolerance Tables Module
C.8.8.12    RTPATSET  RT Patient Setup Module
C.8.8.13    RTFRACT   RT Fraction Scheme Module
C.8.8.14    RTBEAMS   RT Beams Module
C.8.8.15    RT???     RT Brachy Application Setups Module
C.8.8.16    RTAPPRV   Approval Module
C.8.8.17    RT???     RT General Treatment Record Module
C.8.8.18    RT???     RT Treatment Machine Record Module
C.8.8.19    RT???     Measured Dose Reference Record Module
C.8.8.20    RT???     Calculated Dose Reference Record Module
C.8.8.21    RT???     RT Beams Session Record Module
C.8.8.22    RT???     RT Brachy Session Record Module
C.8.8.23    RT???     RT Treatment Summary Record Module
C.8.8.24    RTION???  RT Ion Tolerance Tables Module
C.8.8.25    RTION???  RT Ion Beams Module
C.8.8.26    RTION???  RT Ion Beams Session Record Module

C.8.9.1     PETSER    PET Series Module
C.8.9.2     PETISO    PET Isotope Module
C.8.9.3     PETMGACQ  PET Multi-gated Acquisition Module
C.8.9.4     PETIMG    PET Image Module
C.8.9.5     PETCRV    PET Curve Module (Retired. See PS 3.3 2004.)

C.8.10      HCOPY     Hardcopy Modules (Retired. See PS 3.3 2004.)

C.8.11.1    DXSER     DX Series Module
C.8.11.2    DXANA     DX Anatomy Imaged Module
C.8.11.3    DXIMG     DX Image Module
C.8.11.4    DXDET     DX Detector Module
C.8.11.5    DXPOS     DX Positioning Module

C.8.11.6    MAMSER    Mammography Series Module
C.8.11.7    MAMIMG    Mammography Image Module

C.8.11.8    IORASER   Intra-oral Series Module
C.8.11.9    IORAIMG   Intra-oral Image Module

C.8.12.1    VLIMG     VL Image Module

C.8.12.2    SLDCOORD  Slide Coordinates Module

C.8.13.1    MRIMGH    Enhanced MR Image Module
C.8.13.2              MR Image and Spectroscopy Instance Macro
C.8.13.3              MR Image Description Macro
C.8.13.4    MRSEQ     MR Pulse Sequence Module
C.8.13.5              Enhanced MR Image Functional Group Macros
C.8.13.6    MRSER     MR Series Module
C.8.14.1    MRSPC     MR Spectroscopy Module
C.8.14.2    MRSPCSEQ  MR Spectroscopy Pulse Sequence Module
C.8.14.3    MRSPCGRP  MR Spectroscopy Functional Group Macros
C.8.14.4    MRSPCDAT  MR Spectroscopy Data Module

C.8.15.1    CTSER     CT Series Module
C.8.15.2    CTIMGH    Enhanced CT Image Module

C.8.17.1    OPHPHTSER Ophthalmic Photography Series Module
C.8.17.2    OPHPHTIMG Ophthalmic Photography Image Module
C.8.17.3    OPHPHTPAR Ophthalmic Photographic Parameters Module
C.8.17.4    OPHPHTACQ Ophthalmic Photography Acquisition Parameters Module
C.8.17.5    OCLREGI   Ocular Region Imaged Module
C.8.17.6    OPHTOMSER Ophthalmic Tomography Series Module
C.8.17.7    OPHTOMIMG Ophthalmic Tomography Image Module
C.8.17.8    OPHTOMACQ Ophthalmic Tomography Acquisition Parameters Module
C.8.17.9    OPHTOMPAR Ophthalmic Tomography Parameters Module

C.8.18.1    ???       Stereometric Series Module
C.8.18.2    ???       Stereometric Relationship Module

C.8.19.1    XAXRFSER  XA/XRF Series Module
C.8.19.2    XAXRFIMGH Enhanced XA/XRF Image Module
C.8.19.3    XAXRFACQ  XA/XRF Acquisition Module
C.8.19.4    XR???     X-Ray Image Intensifier module
C.8.19.5    XRDET     X-Ray Detector Module
C.8.19.6              Enhanced XA/XRF Image Functional Group Macros
C.8.19.6.1            XA/XRF Frame Characteristics Macro
C.8.19.6.2            X-Ray Field of View Macro
C.8.19.6.3            X-Ray Exposure Control Sensing Regions Macro
C.8.19.6.4            XA/XRF Frame Pixel Data Properties Macro
C.8.19.6.5            X-Ray Frame Detector Parameters Macro
C.8.19.6.6            X-Ray Calibration Device Usage Macro
C.8.19.6.7            X-Ray Object Thickness Macro
C.8.19.6.8            X-Ray Frame Acquisition Macro
C.8.19.6.9            X-Ray Projection Pixel Calibration Macro
C.8.19.6.10           X-Ray Positioner Macro
C.8.19.6.11           X-Ray Table Position Macro
C.8.19.6.12           X-Ray Collimator Macro
C.8.19.6.13           X-Ray Isocenter Reference System Macro
C.8.19.6.14           X-Ray Geometry Macro
C.8.19.7              XA/XRF Multi-frame Presentation Module

C.8.20.1    SEGSER    Segmentation Series Module
C.8.20.2    SEGIMG    Segmentation Image Module

C.8.21.1    XR???     X-Ray 3D Image Module
C.8.21.2.1  XR???     X-Ray 3D Angiographic Image Contributing Sources Module
C.8.21.2.2  XR???     X-Ray 3D Craniofacial Image Contributing Sources Module
C.8.21.3.1  XR???     X-Ray 3D General Acquisition Macros
C.8.21.3.2  XR???     X-Ray 3D Angiographic Acquisition Module
C.8.21.3.3  XR???     X-Ray 3D Craniofacial Acquisition Module
C.8.21.4    XR???     X-Ray 3D Reconstruction Module

C.9.1       OVL???    Overlay identification module (Retired. See PS 3.3 2004.)
C.9.2       OVL???    Overlay plane module
C.9.3       OVL???    Multi-frame Overlay Module
C.9.4       OVL???    Bi-Plane Overlay Module (Retired)
C.9.5       OVL???    Basic Print Image Overlay Box Module (Retired. See PS 3.3 2004.)

C.10.1      CRVIDF    Curve identification module (Retired. See PS 3.3 2004.) 
C.10.2      CRV       Curve module (Retired. See PS 3.3 2004.)

C.10.3      AUD       Audio module (Retired. See PS 3.3 2004.)

C.10.4      ??? Displayed Area Module
C.10.5      ??? Graphic Annotation Module
C.10.6      ??? Spatial Transformation Module
C.10.7      ??? Graphic Layer Module

C.10.8      WAVIDF    Waveform Identification Module
C.10.9      WAV       Waveform Module
C.10.10     WAVANN    Waveform Annotation Module

C.11.1      ??? Modality LUT Module
C.11.2      ??? VOI LUT Module
C.11.3      ??? LUT Identification Module
C.11.4      ??? Presentation LUT Module
C.11.5      ??? Image Histogram Module
C.11.6      ??? Softcopy Presentation LUT Module
C.11.7      ??? Overlay Activation Module
C.11.8      ??? Softcopy VOI LUT module

C.11.9      PRS??? Presentation Series Module
C.11.10     PRS??? Presentation State Identification Module
C.11.11     PRS??? Presentation State Relationship Module
C.11.12     PRS??? Presentation State Shutter Module
C.11.13     PRS??? Presentation State Mask Module
C.11.14     PRS??? Presentation State Blending Module

C.11.15     ICCPRF    ICC Profile Module

C.12.1      ??? SOP Common Module
C.12.2      ??? Common Instance Reference Module

C.13.1      FLM??? Basic Film Session Presentation Module
C.13.2      FLM??? Basic Film Session Relationship Module
C.13.3      FLM??? Basic Film Box Presentation Module
C.13.4      FLM??? Basic Film Box Relationship Module

C.13.5      IMGBOX??? Image Box Pixel Presentation Module
C.13.6      IMGBOX??? Image Box Relationship Module (Retired. See PS 3.3-1998.)

C.13.7      ??? Basic Annotation Presentation Module

C.13.8      PRTJOB    Print Job Module
C.13.9      PRT       Printer Module

C.13.10     ??? Image Overlay Box Presentation Module (Retired. See PS 3.3-1998.)
C.13.11     ??? Image Overlay Box Relationship Module (Retired. See PS 3.3-1998.)

C.13.12     PRTREQ    Print Request Module (Retired. See PS 3.3 2004.)
C.13.13     PRTCFG    Printer Configuration Module

C.14        ??? Storage Commitment Module

C.15        ??? Queue Management Specific Modules (Retired. See PS 3.3 2004.)

C.16        ??? Stored Print Specific Modules (Retired. See PS 3.3 2004.)

C.17.1      SRDOCSER  SR Document Series Module
C.17.2      SRDOC???  SR Document General Module
C.17.3      SRDOC???  SR Document Content Module

C.17.6.1    ??? Key Object Document Series Module
C.17.6.2    ??? Key Object Document Module

C.19.1      RAWDAT    Raw Data Module

C.20.1      REG??? Spatial Registration Series Module
C.20.2      REG??? Spatial Registration Module
C.20.3      REG??? Deformable Spatial Registration Module

C.21.1      FIDSER    Spatial Fiducials Series Module
C.21.2      FID       Spatial Fiducials Module

C.22.1      ??? Media Creation Management Module

C.23.1      HPRTDEF   Hanging Protocol Definition Module
C.23.2      HPRTENV   Hanging Protocol Environment Module
C.23.3      HPRTDSP   Hanging Protocol Display Module

C.24.1      DOC???    Encapsulated Document Series Module
C.24.2      DOC???    Encapsulated Document Module

C.25.1      ??? Real World Value Mapping Series Module
C.25.2      ??? Real World Value Mapping Module

C.26.1      ??? Product Characteristics Module

C.26.2      SUB??? Substance Approval Module
C.26.3      SUB??? Substance Administration Module
C.26.4      SUB??? Substance Administration Log Module

*/
                                      /*
                                         The DICOM attributes, PS 3.10-2008
                                      */ 
enum{
   DICOM_TYPE1=0x1,                /*
                                         Type 1: 
                                         Such Attributes shall be present 
                                         with an explicit Value in files 
                                         created by File-set Creators and 
                                         File-set Updaters. 
                                         They shall be supported by 
                                         File-set Readers and File-set Updaters; 
                                      */ 
   DICOM_TYPE1C=0x1C,              /* Type 1C: 
                                         Such Attributes shall be present 
                                         with an explicit Value in files 
                                         created by File-set Creators and 
                                         File-set Updaters if the specified condition is met. 
                                         They shall be supported by 
                                         File-set Readers and File-set Updaters.
                                      */
   DICOM_TYPE2=0x2,                /* Type 2: 
                                         Such Attributes shall be present 
                                         with an explicit Value or 
                                         with a zero-length Value if unknown, 
                                         in Files created by File-set Creators 
                                         and File-set Updaters.
                                         They shall be supported by 
                                         File-set Readers and File-set Updaters; 
                                      */
   DICOM_TYPE2C=0x2C,              /* Type 2C: 
                                         Such Attributes shall be present with 
                                         an explicit Value or with a zero-length if unknown, 
                                         in Files created by File-set Creators and File-set Updaters 
                                         if the specified condition is met. 
                                         They shall be supported by File-set Readers and File-set Updaters; 
                                      */
   DICOM_TYPE3=0x3,                /* Type 3: 
                                         Such Attributes may be present with an explicit Value 
                                         or a zero-length Value in Files created by 
                                         File-set Creators and File-set Updaters. 
                                         They may be supported or ignored by Fileset Readers and File-set Updaters.
                                      */
};

                                      /* Media Storage Standard SOP Classes,
                                         PS3.4 Table I.4-1
                                      */ 
struct SDICOMSOP {
   int iClass; const char *pcClass; const char *pcDescription;
};

enum {
   DICOMSOP_MediaStorageDirectoryStorage=1, 
   DICOMSOP_ComputedRadiographyImageStorage,
   DICOMSOP_DigitalXRayImageStorageForPresentation,
   DICOMSOP_DigitalXRayImageStorageForProcessing, 
   DICOMSOP_DigitalMammographyImageStorageForPresentation, 
   DICOMSOP_DigitalMammographyImageStorageForProcessing,
   DICOMSOP_DigitalIntraoralXRayImageStorageForPresentation,
   DICOMSOP_DigitalIntraoralXRayImageStorageForProcessing, 
   DICOMSOP_DigitalIntraoralXRayIODCTImageStorage,
   DICOMSOP_EnhancedCTImageStorage,
   DICOMSOP_UltrasoundMultiframeImageStorage,
   DICOMSOP_MRImageStorage,
   DICOMSOP_EnhancedMRImageStorage,
   DICOMSOP_MRSpectroscopyStorage,
   DICOMSOP_UltrasoundImageStorage,
   DICOMSOP_SecondaryCaptureImageStorage,
   DICOMSOP_MultiframeSingleBitSecondaryCaptureImageStorage,
   DICOMSOP_MultiframeGrayscaleByteSecondaryCaptureImageStorage,
   DICOMSOP_MultiframeGrayscaleWordSecondaryCaptureImageStorage,
   DICOMSOP_MultiframeTrueColorSecondaryCaptureImageStorage, 
   DICOMSOP_12leadECGWaveformStorage,
   DICOMSOP_GeneralECGWaveformStorage,
   DICOMSOP_AmbulatoryECGWaveformStorage,
   DICOMSOP_HemodynamicWaveformStorage,
   DICOMSOP_CardiacElectrophysiologyWaveformStorage, 
   DICOMSOP_BasicVoiceAudioWaveformStorage,
   DICOMSOP_GrayscaleSoftcopyPresentationStateStorage,
   DICOMSOP_ColorSoftcopyPresentationStateStorage,
   DICOMSOP_PseudoColorSoftcopyPresentationStateStorage,
   DICOMSOP_BlendingSoftcopyPresentationStateStorage,
   DICOMSOP_XRayAngiographicImageStorage,
   DICOMSOP_EnhancedXAImageStorage,
   DICOMSOP_XRayRadiofluoroscopicImageStorage,
   DICOMSOP_EnhancedXRFImageStorage,
   DICOMSOP_XRay3DAngiographicImageStorage,
   DICOMSOP_XRay3DCraniofacialImageStorage,
   DICOMSOP_NuclearMedicineImageStorage,
   DICOMSOP_RawDataStorage,
   DICOMSOP_SpatialRegistrationStorage,
   DICOMSOP_SpatialFiducialsStorage,
   DICOMSOP_DeformableSpatialRegistrationStorage,
   DICOMSOP_SegmentationStorage,
   DICOMSOP_RealWorldValueMappingStorage,
   DICOMSOP_VLEndoscopicImageStorage,
   DICOMSOP_VideoEndoscopicImageStorage,
   DICOMSOP_VLMicroscopicImageStorage,
   DICOMSOP_VideoMicroscopicImageStorage,
   DICOMSOP_VLSlideCoordinatesMicroscopicImageStorage,
   DICOMSOP_VLPhotographicImageStorage,
   DICOMSOP_VideoPhotographicImageStorage,
   DICOMSOP_OphthalmicPhotography8BitImageStorage,
   DICOMSOP_OphthalmicPhotography16BitImageStorage,
   DICOMSOP_StereometricRelationshipStorage,
   DICOMSOP_OphthalmicTomographyImageStorage,
   DICOMSOP_BasicTextSR,
   DICOMSOP_EnhancedSR,
   DICOMSOP_ComprehensiveSR,
   DICOMSOP_ProcedureLog,
   DICOMSOP_MammographyCADSR,
   DICOMSOP_KeyObjectSelectionDocument,
   DICOMSOP_ChestCADSR,
   DICOMSOP_XRayRadiationDoseSR,
   DICOMSOP_EncapsulatedPDFStorage,
   DICOMSOP_PositronEmissionTomographyImageStorage,
   DICOMSOP_RTImageStorage,
   DICOMSOP_RTDoseStorage,
   DICOMSOP_RTStructureSetStorage,
   DICOMSOP_RTBeamsTreatmentRecordStorage,
   DICOMSOP_RTPlanStorage,
   DICOMSOP_RTBrachyTreatmentRecordStorage,
   DICOMSOP_RTTreatmentSummaryRecordStorage,
   DICOMSOP_RTIonPlanStorage,
   DICOMSOP_RTIonBeamsTreatmentRecordStorage,
   DICOMSOP_HangingProtocolStorage,
};
                                      /* Transfer Syntax */
struct SDICOMXFER {
   int iClass; const char *pcClass; const char *pcDescription;
};

enum { 
   DICOMXFER_IMPLICITLE=1, 
   DICOMXFER_IMPLICITBE, 
   DICOMXFER_EXPLICITLE, 
   DICOMXFER_EXPLICITBE
};
                                      /** \struct SDICOMHEADER
                                          \brief The DICOM header.
                                      */ 
struct SDICOMHEADER {
   uint8_t bPreAmble[128];            /**< 128 bytes (zero) */
   char    cPrefix[4];                /**< "DICM" */
};
                                      /* defined VRs */
enum { 
//!!! endian issue
   DICOMVR_AE=('A'<<8)+'E',        /* Application Entity, characters, 16 byte max  */
   DICOMVR_AS=('A'<<8)+'S',        /* Age String */
   DICOMVR_AT=('A'<<8)+'T',        /* Attribute Tag */
   DICOMVR_CS=('C'<<8)+'S',        /* Code String */
   DICOMVR_DA=('D'<<8)+'A',        /* Date */
   DICOMVR_DS=('D'<<8)+'S',        /* Decimal String */
   DICOMVR_DT=('D'<<8)+'T',        /* Date Time */
   DICOMVR_FL=('F'<<8)+'L',        /* Floating Point Single */
   DICOMVR_FD=('F'<<8)+'D',        /* Floating Point Double */
   DICOMVR_IS=('I'<<8)+'S',        /* Integer String */
   DICOMVR_LO=('L'<<8)+'O',        /* Long String */
   DICOMVR_LT=('L'<<8)+'T',        /* Long Text */
   DICOMVR_OB=('O'<<8)+'B',        /* Other Byte String */ 
   DICOMVR_OF=('O'<<8)+'F',        /* Other Float String */ 
   DICOMVR_OW=('O'<<8)+'W',        /* Other Word String */ 
   DICOMVR_PN=('P'<<8)+'N',        /* Person Name */
   DICOMVR_SH=('S'<<8)+'H',        /* Short String 16 char's max */
   DICOMVR_SL=('S'<<8)+'L',        /* Signed Long */
   DICOMVR_SQ=('S'<<8)+'Q',        /* Sequence of Items */
   DICOMVR_SS=('S'<<8)+'S',        /* Signed Short */
   DICOMVR_ST=('S'<<8)+'T',        /* Short Text */
   DICOMVR_TM=('T'<<8)+'M',        /* Time */
   DICOMVR_UI=('U'<<8)+'I',        /* Unique Identifier */
   DICOMVR_UL=('U'<<8)+'L',        /* Unsigned Long */
   DICOMVR_UN=('U'<<8)+'N',        /* Unknown */
   DICOMVR_US=('U'<<8)+'S',        /* Unsigned Short */
   DICOMVR_UT=('U'<<8)+'T',        /* Unlimited Text */
};

                                      /* overlay 16bit int with 2 chars */
union UNDICOMVR { uint16_t ui16; char c2[2]; };
                                      /** \struct SDICOMDATAELEMENTHEADERVR
                                          \brief Data element with VR.
                                      */ 
struct SDICOMDATAELEMENTHEADERVR {
   uint16_t ui16GN, ui16EN;           /**< group number, element number */
   union UNDICOMVR unVR;           /**< value representation */
   uint16_t ui16VL;                   /**< value length */
};
                                      /** \struct SDICOMDATAELEMENTHEADERVR32
                                          \brief Data element with VR and 32bit VL.
                                      */ 
struct SDICOMDATAELEMENTHEADERVR32 {
   uint16_t ui16GN, ui16EN;           /**< group number, element number */
   union UNDICOMVR unVR;           /**< value representation */
   uint16_t ui16Reserved;             /**< reserved for DICOM */
   uint32_t ui32VL;                   /**< value length */
};
/*

   Functions
   ---------
   
   dicomDataElementHeaderVR32Inspect( headervr32, flag )
                             o inspect 
                             o to test
                             o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomDataElementHeaderVR32Inspect( struct SDICOMDATAELEMENTHEADERVR32 *, unsigned int );

                                      /** \struct SDICOMDATAELEMENTHEADER
                                          \brief Data element, implicit VR.
                                      */ 
struct SDICOMDATAELEMENTHEADER {
   uint16_t ui16GN, ui16EN;           /**< group number, element number */
   uint32_t ui32VL;                   /**< value length */
};


                                      /** \struct SDICOMDATAELEMENT
                                          \brief A DICOM data element node.
                                      */ 
struct SDICOMDATAELEMENT {
   void *pvDEH;                       /**< reference to original data element header */
   void *pvDED;                       /**< reference to original data element data */
   struct SDICOMDATAELEMENTHEADERVR32 sDEH; /**< processed data element header */
                                      /* processed entries: */
   double dd;                         /**< generic double FP */                               
   float rr;                                   
   float *pr;
   double *pd;
    int32_t *pi32;
   uint16_t *pui16;
    int32_t i32;
   uint32_t ui32;
    int16_t i16;                      
   uint16_t ui16;
   uint16_t ui16v[2];                 /**< fixed short [2] */
     
   unsigned int nn;                   /**< # elements in a value array */
                   
   unsigned int nDS, mDS;             /**< current/max # of embedded data sets */
#define DICOM_DATASETINC 32        /* DS table increment */   
   struct SDICOMDATASET *psDS;        /**< [nDS] data sets */
};
                                      /* table increment macro */
#define DICOMTABLEINCREMENT(rc,base,Elm,INC) \
         if ( (base)->n##Elm >= (base)->m##Elm ) {\
            void *pv = NULL;\
            (rc) = -1;\
            pv = realloc( (base)->ps##Elm, (ssize_t) ((base)->m##Elm+INC) * sizeof(*((base)->ps##Elm)) );\
            if ( pv != NULL ) {\
               (base)->ps##Elm = pv;\
               memset( (base)->ps##Elm + (base)->m##Elm, 0, INC * sizeof(*((base)->ps##Elm)));\
               (base)->m##Elm += INC;\
               (rc) = 0;\
            }\
         }                                

                                      /* debug output macro */
#define  DICOMDATAELEMENTDEBUG(de) \
         { fprintf(stderr," %*.*s\n", (de)->sDEH.ui32VL, (de)->sDEH.ui32VL, (char *) (de)->pvDED ); }

                                      /* auxiliary macro to assign (nested) data element => attribute */
#define  DICOMDATAELEMENT2ATTR(path,stem,attr,level)\
      case DICOM##attr:\
         (path).ps##stem[i##level##DS].psDE##attr = & psDE##level[i##level##DE];\
      break;
                                      /* auxiliary macro to assign (nested) data element => attribute */
#define  DICOMDE2ATTRCASE(path,stem,attr,level)
/*
#if level>0
#define  DICOMDE2ATTRCASE(path,stem,attr,level)\
      case DICOM##attr:\
         (path).ps##stem[i##level##DS].psDE##attr = & psDE##level[i##level##DE];\
      break;
#else
#define  DICOMDE2ATTRCASE(path,stem,attr,level)\
      case DICOM##attr:\
         (path).psDE##attr = & psDE##level[i##level##DE];\
      break;\
#endif
*/
                                      /* auxiliary macro  */
#define DICOMSEQUENCE1BEG(path,stem)\
fprintf( stderr, "(%4.4X,%4.4X) nDS=%d\n", psDS0->psDE[i0DE].sDEH.ui16GN, psDS0->psDE[i0DE].sDEH.ui16EN, psDS0->psDE[i0DE].nDS );\
         if ( psDS0->psDE[i0DE].nDS > 0 ) {\
            if ( (\
            (path).ps##stem = calloc( (size_t) psDS0->psDE[i0DE].nDS, sizeof(*((path).ps##stem)) )\
                 ) == NULL ) {\
               sCE.iRC = ERRDICOM_SYS; goto gError;\
            }\
            (path).n##stem = psDS0->psDE[i0DE].nDS;\
            for ( i1DS=0; i1DS<psDS0->psDE[i0DE].nDS; i1DS++ ) {\
               psDE1 = psDS0->psDE[i0DE].psDS[i1DS].psDE;\
fprintf( stderr, "(%4.4X,%4.4X) nDE=%d\n", NULL, NULL, psDS0->psDE[i0DE].psDS[i1DS].nDE );\
               for ( i1DE=0; i1DE<psDS0->psDE[i0DE].psDS[i1DS].nDE; i1DE++ ) {\
fprintf( stderr, "(%4.4X,%4.4X) nDS=%d\n", psDE1[i1DE].sDEH.ui16GN, psDE1[i1DE].sDEH.ui16EN, psDE1[i1DE].nDS );\
                  switch( (uint32_t) (psDE1[i1DE].sDEH.ui16GN << 16) | psDE1[i1DE].sDEH.ui16EN ) {

                                      /* auxiliary macro  */
#define DICOMSEQUENCE1END(path,stem)\
                  default: break;\
                  } /* tag level 1 */\
               } /* DE level 1 */\
            }/* DS level 1 */\
         }/* Datasets embedded */
         
         
                                      /* inspect output macro */
#define DICOMELEMENTDATAINSPECT(width,path,name)\
      if ( (path).psDE##name != NULL )\
      switch( (path).psDE##name->sDEH.unVR.ui16 ) { int ii;\
      case DICOMVR_AE:\
      case DICOMVR_AS:\
      case DICOMVR_CS:\
      case DICOMVR_DA:\
      case DICOMVR_DT:\
      case DICOMVR_LO:\
      case DICOMVR_LT:\
      case DICOMVR_OB:\
      case DICOMVR_PN:\
      case DICOMVR_SH:\
      case DICOMVR_ST:\
      case DICOMVR_TM:\
      case DICOMVR_UI:\
      case DICOMVR_UN:\
      case DICOMVR_UT:\
   fprintf( stderr, "%*.*s=%*.*s\n", width,width, Str(name), (path).psDE##name->sDEH.ui32VL, (path).psDE##name->sDEH.ui32VL, (path).psDE##name->pvDED );\
      break;\
      case DICOMVR_AT:\
   fprintf( stderr, "%*.*s=%d %d\n", width,width, Str(name), (path).psDE##name->ui16v[0], (path).psDE##name->ui16v[1] );\
      break;\
      case DICOMVR_DS:\
   fprintf( stderr, "%*.*s=", width,width, Str(name) );\
         if ( (path).psDE##name->pd != NULL )\
         for ( ii=0; ii<(path).psDE##name->nn; ii++ ) {\
   fprintf( stderr, "%G ", (path).psDE##name->pd[ii] );\
         }\
         else\
   fprintf( stderr, "%G ", (path).psDE##name->dd );\
   fprintf( stderr, "\n" );\
      break;\
      case DICOMVR_FL:\
   fprintf( stderr, "%*.*s=%G\n", width,width, Str(name), (path).psDE##name->rr );\
      break;\
      case DICOMVR_FD:\
   fprintf( stderr, "%*.*s=%G\n", width,width, Str(name), (path).psDE##name->dd );\
      break;\
      case DICOMVR_IS:\
   fprintf( stderr, "%*.*s=", width,width, Str(name) );\
         if ( (path).psDE##name->pi32 != NULL )\
         for ( ii=0; ii<(path).psDE##name->nn; ii++ ) {\
   fprintf( stderr, "%d ", (path).psDE##name->pi32[ii] );\
         }\
         else\
   fprintf( stderr, "%d ", (path).psDE##name->i32 );\
   fprintf( stderr, "\n" );\
      break;\
      case DICOMVR_OF:\
   fprintf( stderr, "%*.*s=", width,width, Str(name) );\
         for ( ii=0; ii<(path).psDE##name->nn; ii++ ) { \
   fprintf( stderr, "%G ", (path).psDE##name->pr[ii] );\
         }\
   fprintf( stderr, "\n" );\
      break; \
      case DICOMVR_OW:\
   fprintf( stderr, "%*.*s=", width,width, Str(name) );\
         for ( ii=0; ii<(path).psDE##name->nn; ii++ ) { \
   fprintf( stderr, "%d ", (path).psDE##name->pui16[ii] );\
         }\
   fprintf( stderr, "\n" );\
      break;\
      case DICOMVR_SL:\
   fprintf( stderr, "%*.*s=%d\n", width,width, Str(name), (path).psDE##name->i32 );\
      break; \
      case DICOMVR_SS:\
   fprintf( stderr, "%*.*s=%d\n", width,width, Str(name), (path).psDE##name->i16 );\
      break; \
      case DICOMVR_UL:\
   fprintf( stderr, "%*.*s=%u\n", width,width, Str(name), (path).psDE##name->ui32 );\
      break; \
      case DICOMVR_US:\
   fprintf( stderr, "%*.*s=%u\n", width,width, Str(name), (path).psDE##name->ui16 );\
      break; \
      default:\
      break;\
      }                                


/*

   Functions
   ---------
   
   dicomDataElementFree   ( dcmdataelement, flag )
                             o releases resources
                             o to test
                             o calls:
                               o dicomDatasetFree()
                               o dicomDataElementInit()
                            
   dicomDataElementFromBuf( dcmdataelement, map, syntax, flag )
                             o fill data element from buffer (in map)
                             o to test
                             o to do
                               o implicit VR
                               o unVR endian issue
                               o length from SQ delimiter
                             o calls:
                               o dicomDatasetInit()
                               o dicomDatasetRead
                               o dicomDataElementInit
                               o dicomDataElementFree
                        
   dicomDataElementInit   ( dcmdataelement, flag )
                             o initialize
                             o to test
                             o calls:
                         
   dicomDataElementInspect( dcmdataelement, flag )
                             o inspect 
                             o to test
                             o to complete
                             o calls:
                               o dicomDataSetInspect
                               o dicomDataElementHeaderVR32Inspect
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomDataElementFree   ( struct SDICOMDATAELEMENT *, unsigned int );
struct SCMDERR dicomDataElementFromBuf( struct SDICOMDATAELEMENT *, struct SUMMAP *, unsigned int, unsigned int );
struct SCMDERR dicomDataElementInit   ( struct SDICOMDATAELEMENT *, unsigned int );
struct SCMDERR dicomDataElementInspect( struct SDICOMDATAELEMENT *, unsigned int );
#define DICOMDATAELEMENT_LISTTAGS 0x00010000 /* list tags */

                                      /** \struct SDICOMDATASET
                                          \brief A DICOM data set node.
                                      */ 
struct SDICOMDATASET {
   int nDE, mDE;                      /**< current/max # of data elements */
#define DICOMDATAELEMENT_INC 32    /* DE table increment */   
   struct SDICOMDATAELEMENT *psDE; /**< [nDE] data elements */
};
/*

   Functions
   ---------
   
   dicomDatasetFree   ( dcmdataset, flag )
                      o releases resources
                      o to test
                      o calls:
                        o dicomDataElementFree
                        o dicomDatasetInit()
                            
   dicomDatasetInit   ( dcmdataset, flag )
                      o initialize
                      o to test
                      o calls:
                         
   dicomDatasetInspect( dcmdataset, flag )
                      o inspect 
                      o to test
                      o calls:
                        o dicomDataElementInspect
                         
   dicomDatasetRead   ( dcmdataset, map, flag )
                      o reads
                      o to test
                      o to complete
                        o transfer syntax info
                      o calls:
                        o dicomDataElementInit
                        o dicomDataElementFromBuf
                        o dicomDatasetInit
                        o dicomDatasetFree
*/
                                      /* function prototypes */
struct SCMDERR dicomDatasetFree   ( struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomDatasetInit   ( struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomDatasetInspect( struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomDatasetRead   ( struct SDICOMDATASET *, struct SUMMAP *, unsigned int, unsigned int );
#define DICOMDATASETREAD_LISTTAGS 0x00010000 /* list tags */

                                      /** \struct SDICOMFILE
                                          \brief A DICOM file node.
                                      */ 
struct SDICOMFILE {
   struct SDICOMHEADER sHD;           /**< file header */
   struct SUMMAP sM;                  /**< file map */
   struct SDICOMDATASET sDS;
   uint8_t bvVersion[2];              /**< identifies version of File Meta Information */
   unsigned int uiSyntax;             /**< transfer syntax found in Meta Info */
   unsigned int uiSOPClass;           /**< storage class found in Meta Info */
};
         
/*

   Functions
   ---------
   
   dicomFileFree   ( dcmfile, flag )
                   o releases resources
                   o to test
                   o calls:
                     o dicomFileInit()
                     o dicomDataSetFree()
                            
   dicomFileInit   ( dcmfile, flag )
                   o initialize
                   o to test
                   o calls:
                     o dicomDataSetInit()
                         
   dicomFileInspect( dcmfile, flag )
                   o inspect 
                   o to test
                   o calls:
                     o dicomDataSetInspect()
                         
   dicomFileRead   ( dcmfile, filename, flag )
                   o reads file
                   o to test
                   o to complete
                     o decode transfer syntax
                   o calls:
                     o dicomDataElementInit
                     o dicomDataElementFree
                     o dicomDataElementFromBuf
                     o dicomFileInit
                     o dicomFileFree
                     o dicomDataSetRead
                     
*/
                                      /* function prototypes */
struct SCMDERR dicomFileFree   ( struct SDICOMFILE *, unsigned int );
struct SCMDERR dicomFileInit   ( struct SDICOMFILE *, unsigned int );
struct SCMDERR dicomFileInspect( struct SDICOMFILE *, unsigned int );
struct SCMDERR dicomFileRead   ( struct SDICOMFILE *, const char *, unsigned int );

#define DICOMFILEREAD_LISTTAGS 0x00010000 /* list tags */

struct SDICOM _dicomTest( unsigned int );
                                      /* error handling */
int dicomError( struct SCMDERR *, unsigned int );

enum { 
   ERRDICOM_SYS=1,
   ERRDICOM_BADADDR,
   ERRDICOM_FILESHORT,
   ERRDICOM_BADXFERSYNTAX,
   ERRDICOM_FILEEMPTY,
   ERRDICOM_BADVR,
   ERRDICOM_FILEMETAINFOMISSING,
   ERRDICOM_FILEMETAINFOINDATA,
   ERRDICOM_NOTDICOM,   
   ERRDICOM_BADVERSION,
   ERRDICOM_BADMEDIASTORAGECLASS,
   ERRDICOM_BADSOPCLASS,
   ERRDICOM_BADDECIMAL,
   ERRDICOM_BADINTEGER,
};

#endif
