#ifndef DCMDIR_H
#define DCMDIR_H
/** \file

   Definitions to handle DICOM Basic Directories.

*/
/*

   Copyright  2005-2011 M.Kraemer, GSI, D-64291 Darmstadt

   $Log$

*/
#include "dcmbasic.h"
#include "dcmpat.h"
#include "dcmstudy.h"
#include "dcmseries.h"
#include "dcmimage.h"

                                      /** \struct SDICOMDIRECTORYRECORD
                                          \brief Directory Record.
                                      */ 
                                      /* PS 3.3 Annex F
                                         Table F.3-2 
                                      */
struct SDICOMDIRECTORYRECORD {
   struct SDICOMDATAELEMENT 
          *psDEOffsetoftheNextDirectoryRecord
         ,*psDERecordInuseFlag
         ,*psDEOffsetofReferencedLowerLevelDirectoryEntity
         ,*psDEDirectoryRecordType
         ,*psDEPrivateRecordUID
         ,*psDEReferencedFileID
         ,*psDEReferencedSOPClassUIDinFile
         ,*psDEReferencedSOPInstanceUIDinFile
         ,*psDEReferencedTransferSyntaxUIDinFile
         ,*psDEReferencedRelatedGeneralSOPClassUIDinFile
         ;
   struct SDICOMDATAELEMENT 
          *psDESpecificCharacterSet
         ,*psDEModality
         ,*psDESeriesInstanceUID
         ,*psDESeriesNumber
         ,*psDEInstanceNumber
         ;

   int nIPM;
   struct SDICOMIMAGEPIXELMACRO *psIPM;
         
   struct SDICOMPATIENTMODULE sPM;      
   struct SDICOMSTUDYMODULE   sStdM;      
   struct SDICOMSERIESMODULE  sSerM;      
   struct SDICOMIMAGEMODULE   sIM;      

   struct SDICOMFILE sF;
};

                                      /* directory record types */
enum {
   DICOMRECORD_NONE,
   DICOMRECORD_PATIENT, 
   DICOMRECORD_STUDY, 
   DICOMRECORD_SERIES,
   DICOMRECORD_IMAGE, 
   DICOMRECORD_RTDOSE, 
   DICOMRECORD_RTSTRUCTURESET,
   DICOMRECORD_RTPLAN, 
   DICOMRECORD_RTTREATRECORD, 
   DICOMRECORD_PRESENTATION, 
   DICOMRECORD_WAVEFORM,
   DICOMRECORD_SRDOCUMENT,
   DICOMRECORD_KEYOBJECTDOC,
   DICOMRECORD_SPECTROSCOPY, 
   DICOMRECORD_RAWDATA, 
   DICOMRECORD_REGISTRATION, 
   DICOMRECORD_FIDUCIAL, 
   DICOMRECORD_HANGINGPROTOCOL, 
   DICOMRECORD_ENCAPDOC, 
   DICOMRECORD_HL7STRUCDOC, 
   DICOMRECORD_VALUEMAP,
   DICOMRECORD_STEREOMETRIC,
   DICOMRECORD_PRIVATE,
};
/*

   Functions
   ---------
   
   dicomDirectoryRecordFree   ( dr, flag )
                              o releases resources
                            3 o to do
                              o calls:

   dicomDirectoryRecordFromDS ( dr, ds, flag )
                              o build record from DICOM dataset
                            4 o to do
                              o calls:

   dicomDirectoryRecordInit   ( dr, flag )
                              o initialize
                            2 o to do
                              o calls:

   dicomDirectoryRecordInspect( dr, flag )
                              o inspect 
                            1 o to do
                              o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomDirectoryRecordFree    ( struct SDICOMDIRECTORYRECORD *, unsigned int );
struct SCMDERR dicomDirectoryRecordFromFile( struct SDICOMDIRECTORYRECORD *, struct SDICOMFILE *, unsigned int );
struct SCMDERR dicomDirectoryRecordInit    ( struct SDICOMDIRECTORYRECORD *, unsigned int );
struct SCMDERR dicomDirectoryRecordInspect ( struct SDICOMDIRECTORYRECORD *, unsigned int );

struct SDICOMDIRECTORYRECORD _dicomDirectoryRecordTest( unsigned int );



                                      /** \struct SDICOMDIRECTORYINFORMATIONMODULE
                                          \brief Directory Information Module.
                                      */ 
                                      /* PS 3.3 Annex F
                                         F.3.2.2 Directory Information Module.
                                         Table F.3-3
                                      */
struct SDICOMDIRECTORYINFORMATIONMODULE {
   struct SDICOMDATAELEMENT 
          *psDEFirstRecord
         ,*psDELastRecord
         ,*psDEConsistency
         ;
   int nDR;
   struct SDICOMDIRECTORYRECORD *psDR;      
};


                                      /** \struct SDICOMFILESETIDMODULE
                                          \brief File-set Identification Module.
                                      */ 
                                      /* PS 3.3 Annex F
                                         Table  F.3-2 File-set Identification Module
                                      */
struct SDICOMFILESETIDMODULE {
   struct SDICOMDATAELEMENT 
          *psDEFilesetID
         ,*psDEDescriptorFileID
         ,*psDEDescriptorCharSet
         ;
};
                                      /** \struct SDICOMBASICDIRECTORY
                                          \brief Holds Basic Directory.
                                      */ 
                                      /* PS 3.3 Annex F
                                         Table  F.3-1 basic directory IOD modules
                                      */
struct SDICOMBASICDIRECTORY {
   struct SDICOMFILESETIDMODULE sFSIM;
   struct SDICOMDIRECTORYINFORMATIONMODULE sDIM;
};
/*

   Functions
   ---------
   
   dicomBasicDirectoryFree    ( bd, flag )
                              o releases resources
                            3 o to do
                              o calls:

   dicomBasicDirectoryFromFile( bd, dcmfile, flag )
                              o build Basic Directory from DICOM file
                            4 o to test
                              o calls:

   dicomBasicDirectoryInit    ( bd, flag )
                              o initialize
                            2 o to do
                              o calls:

   dicomBasicDirectoryInspect ( bd, flag )
                              o inspect 
                            1 o to test
                              o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomBasicDirectoryFree    ( struct SDICOMBASICDIRECTORY *, unsigned int );
struct SCMDERR dicomBasicDirectoryFromFile( struct SDICOMBASICDIRECTORY *, struct SDICOMFILE *, unsigned int );
struct SCMDERR dicomBasicDirectoryInit    ( struct SDICOMBASICDIRECTORY *, unsigned int );
struct SCMDERR dicomBasicDirectoryInspect ( struct SDICOMBASICDIRECTORY *, unsigned int );

struct SDICOM _dicomBasicDirectoryTest( unsigned int );

#endif
