#ifndef DCMIMAGE_H
#define DCMIMAGE_H
/** \file

   Definitions to handle DICOM Image Modules.

*/
/*

   Copyright  2005-2011 M.Kraemer, GSI, D-64291 Darmstadt

   $Log$

*/
#include "dcmbasic.h"
#include "dcmmacro.h"

                                      /* PS 3.3
                                         C.7.6.3 
                                         Table C.7-11a Image Pixel Module Attributes
                                      */
                                      /** \struct SDICOMIMAGEPIXELMODULE
                                          \brief .
                                      */ 
struct SDICOMIMAGEPIXELMODULE {
   struct SDICOMDATAELEMENT 
          *psDEPixelDataProviderURL
         ,*psDEPixelPaddingRangeLimit
         ;
};
                                      /* PS 3.3
                                         C.7.6.3 
                                         Table C.7-11b Image Pixel Macro Attributes
                                      */
                                      /** \struct SDICOMIMAGEPIXELMACRO
                                          \brief .
                                      */ 
struct SDICOMIMAGEPIXELMACRO {
   struct SDICOMDATAELEMENT 
          *psDESamplesperPixel
         ,*psDEPhotometricInterpretation
         ,*psDERows
         ,*psDEColumns
         ,*psDEBitsAllocated
         ,*psDEBitsStored
         ,*psDEHighBit
         ,*psDEPixelRepresentation
         ,*psDEPixelData
         ,*psDEPlanarConfiguration
         ,*psDEPixelAspectRatio
         ,*psDESmallestImagePixelValue
         ,*psDELargestImagePixelValue
         ,*psDERedPaletteColorLookupTableDescriptor
         ,*psDEGreenPaletteColorLookupTableDescriptor
         ,*psDEBluePaletteColorLookupTableDescriptor
         ,*psDERedPaletteColorLookupTableData
         ,*psDEGreenPaletteColorLookupTableData
         ,*psDEBluePaletteColorLookupTableData
         ,*psDEICCProfile
         ;
};
/*

   Functions
   ---------
   
   dicomImagePixelMacroFree   ( ipm, flag )
                              o releases resources
                              o to do
                              o calls:
                            
   dicomImagePixelMacroFromDS ( ipm, dicomds, flag )
                              o fill image pixel macro attributes from DICOM dataset
                              o to do
                              o calls:
   
   dicomImagePixelMacroInit   ( ipm, flag )
                              o initialize
                              o to do
                              o calls:
                         
   dicomImagePixelMacroInspect( ipm, flag )
                              o inspect 
                              o to do
                              o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomImagePixelMacroFree   ( struct SDICOMIMAGEPIXELMACRO *, unsigned int );
struct SCMDERR dicomImagePixelMacroFromDS ( struct SDICOMIMAGEPIXELMACRO *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomImagePixelMacroInit   ( struct SDICOMIMAGEPIXELMACRO *, unsigned int );
struct SCMDERR dicomImagePixelMacroInspect( struct SDICOMIMAGEPIXELMACRO *, unsigned int );

struct SDICOMIMAGEPIXELMACRO _dicomImagePixelMacroTest( unsigned int );



                                      /* PS 3.3
                                         C.7.6.1 
                                         Table C.7-9 General Image IE Modules
                                      */

                                      /** \struct SDICOMREFERENCEDIMAGE
                                          \brief .
                                      */ 
struct SDICOMREFERENCEDIMAGE {
   struct SDICOMIMAGESOPINSTANCEREFERENCE sISIR /* Table 10-3 */; 
                                      /* Purpose of Reference Code Sequence */
   int nPRC;
   struct SDICOMCODESEQUENCE *psPRC; 
};
/*

   Functions
   ---------
   
   dicomReferencedImageFree    ( ri, flag )
                              o releases resources
                              o to do
                              o calls:
                            
   dicomReferencedImageFromDS  ( ri, dicomds, flag )
                              o build Referenced Image from DICOM file
                              o to do
                              o calls:
   
   dicomReferencedImageInit    ( ri, flag )
                              o initialize
                              o to do
                              o calls:
                         
   dicomReferencedImageInspect ( ri, flag )
                              o inspect 
                              o to do
                              o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomReferencedImageFree   ( struct SDICOMREFERENCEDIMAGE *, unsigned int );
struct SCMDERR dicomReferencedImageFromDS ( struct SDICOMREFERENCEDIMAGE *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomReferencedImageInit   ( struct SDICOMREFERENCEDIMAGE *, unsigned int );
struct SCMDERR dicomReferencedImageInspect( struct SDICOMREFERENCEDIMAGE *, unsigned int );

struct SDICOMREFERENCEDIMAGE _dicomReferencedImageTest( unsigned int );


                                      /** \struct SDICOMSOURCEIMAGE
                                          \brief .
                                      */ 
struct SDICOMSOURCEIMAGE {
   struct SDICOMIMAGESOPINSTANCEREFERENCE sISIR /* Table 10-3 */; 
   struct SDICOMDATAELEMENT 
          *psDESpatialLocationsPreserved
         ,*psDEPatientOrientation
         ; 
                                      /* Purpose of Reference Code Sequence */
   int nPRC;
   struct SDICOMCODESEQUENCE *psPRC; 
};
/*

   Functions
   ---------
   
   dicomSourceImageFree    ( si, flag )
                              o releases resources
                              o to do
                              o calls:
                            
   dicomSourceImageFromDS  ( si, dicomds, flag )
                              o build Source Image from DICOM file
                              o to do
                              o calls:
   
   dicomSourceImageInit    ( si, flag )
                              o initialize
                              o to do
                              o calls:
                         
   dicomSourceImageInspect ( si, flag )
                              o inspect 
                              o to do
                              o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomSourceImageFree   ( struct SDICOMSOURCEIMAGE *, unsigned int );
struct SCMDERR dicomSourceImageFromDS ( struct SDICOMSOURCEIMAGE *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomSourceImageInit   ( struct SDICOMSOURCEIMAGE *, unsigned int );
struct SCMDERR dicomSourceImageInspect( struct SDICOMSOURCEIMAGE *, unsigned int );

struct SDICOMSOURCEIMAGE _dicomSourceImageTest( unsigned int );


                                      /** \struct SDICOMREFERENCEDINSTANCE
                                          \brief .
                                      */ 
struct SDICOMREFERENCEDINSTANCE {
   struct SDICOMSOPINSTANCEREFERENCE sSIR; 
                                      /* Purpose of Reference Code Sequence */
   int nPRC;
   struct SDICOMCODESEQUENCE *psPRC; 
};
/*

   Functions
   ---------
   
   dicomReferencedInstanceFree   ( ri, flag )
                              o releases resources
                              o to test
                              o calls:
                            
   dicomReferencedInstanceFromDS ( ri, dicomds, flag )
                              o build Referenced Instance from DICOM file
                              o to test
                              o calls:
   
   dicomReferencedInstanceInit   ( ri, flag )
                              o initialize
                              o to test
                              o calls:
                         
   dicomReferencedInstanceInspect( ri, flag )
                              o inspect 
                              o to test
                              o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomReferencedInstanceFree   ( struct SDICOMREFERENCEDINSTANCE *, unsigned int );
struct SCMDERR dicomReferencedInstanceFromDS ( struct SDICOMREFERENCEDINSTANCE *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomReferencedInstanceInit   ( struct SDICOMREFERENCEDINSTANCE *, unsigned int );
struct SCMDERR dicomReferencedInstanceInspect( struct SDICOMREFERENCEDINSTANCE *, unsigned int );

struct SDICOMREFERENCEDINSTANCE _dicomReferencedInstanceTest( unsigned int );

                                      /** \struct SDICOMGENERALIMAGEMODULE
                                          \brief Holds General Image Module Attributes.
                                      */ 
                                      /* PS 3.3
                                         C.7.6.1 
                                         Table C.7-9 General Image Module Attributes
                                      */
struct SDICOMGENERALIMAGEMODULE {
   struct SDICOMDATAELEMENT 
          *psDEInstanceNumber
         ,*psDEPatientOrientation
         ,*psDEContentDate
         ,*psDEContentTime
         ,*psDEImageType
         ,*psDEAcquisitionNumber
         ,*psDEAcquisitionDate
         ,*psDEAcquisitionTime
         ,*psDEAcquisitionDateTime
         ,*psDEDerivationDescription
         ,*psDEImagesinAcquisition
         ,*psDEImageComments
         ,*psDEQualityControlImage
         ,*psDEBurnedInAnnotation
         ,*psDELossyImageCompression
         ,*psDELossyImageCompressionRatio
         ,*psDELossyImageCompressionMethod
         ,*psDEPresentationLUTShape
         ,*psDEIrradiationEventUID
         ;
                                      /* Referenced Image Sequence */
   int nRImg;
   struct SDICOMREFERENCEDIMAGE *psRImg; 
                                      /* Derivation Code Sequence */
   int nDC;
   struct SDICOMCODESEQUENCE *psDC; 
                                      /* Source Image Sequence */
   int nSImg;
   struct SDICOMSOURCEIMAGE *psSImg; 
                                      /* Referenced Instance Sequence */
   int nRInst;
   struct SDICOMREFERENCEDINSTANCE *psRInst; 
                                      /* Icon Image Sequence */
   int nIImg;
   struct SDICOMIMAGEPIXELMACRO *psIImg; 
};
/*

   Functions
   ---------
   
   dicomGeneralImageModuleFree    ( gim, flag )
                              o releases resources
                              o to do
                              o calls:
                            
   dicomGeneralImageModuleFromDS  ( gim, dicomds, flag )
                              o build image module from DICOM file
                              o to test
                              o calls:
   
   dicomGeneralImageModuleInit    ( gim, flag )
                              o initialize
                              o to do
                              o calls:
                         
   dicomGeneralImageModuleInspect ( gim, flag )
                              o inspect 
                              o to do
                              o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomGeneralImageModuleFree   ( struct SDICOMGENERALIMAGEMODULE *, unsigned int );
struct SCMDERR dicomGeneralImageModuleFromDS ( struct SDICOMGENERALIMAGEMODULE *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomGeneralImageModuleInit   ( struct SDICOMGENERALIMAGEMODULE *, unsigned int );
struct SCMDERR dicomGeneralImageModuleInspect( struct SDICOMGENERALIMAGEMODULE *, unsigned int );

struct SDICOMGENERALIMAGEMODULE _dicomGeneralImageModuleTest( unsigned int );

                                      /* PS 3.3
                                         C.7.6.2 
                                         Table C.7-10 Image Plane Module Attributes
                                      */
                                      /** \struct SDICOMIMAGEPLANEMODULE
                                          \brief .
                                      */ 
struct SDICOMIMAGEPLANEMODULE {
   struct SDICOMDATAELEMENT 
          *psDEPixelSpacing
         ,*psDEImageOrientationPatient
         ,*psDEImagePositionPatient
         ,*psDESliceThickness
         ,*psDESliceLocation
         ;
};


                                      /** \struct SDICOMIMAGEMODULE
                                          \brief Holds Image Module Attributes.
                                      */ 
                                      /* PS 3.3
                                         C.7.6 Common Image IE Modules
                                      */
struct SDICOMIMAGEMODULE {
   struct SDICOMGENERALIMAGEMODULE sGIM;
   struct SDICOMIMAGEPLANEMODULE sIPM;
};
/*

   Functions
   ---------
   
   dicomImageModuleFree    ( im, flag )
                              o releases resources
                              o to do
                              o calls:
                            
   dicomImageModuleFromDS  ( im, dicomds, flag )
                              o build image module from DICOM file
                              o to test
                              o calls:
   
   dicomImageModuleInit    ( im, flag )
                              o initialize
                              o to do
                              o calls:
                         
   dicomImageModuleInspect ( im, flag )
                              o inspect 
                              o to do
                              o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomImageModuleFree   ( struct SDICOMIMAGEMODULE *, unsigned int );
struct SCMDERR dicomImageModuleFromDS ( struct SDICOMIMAGEMODULE *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomImageModuleInit   ( struct SDICOMIMAGEMODULE *, unsigned int );
struct SCMDERR dicomImageModuleInspect( struct SDICOMIMAGEMODULE *, unsigned int );

struct SDICOMIMAGEMODULE _dicomImageModuleTest( unsigned int );

#endif
