#ifndef DCMMACRO_H
#define DCMMACRO_H
/** \file

   Definitions to handle DICOM Macros.

*/
/*

   Copyright  2005-2011 M.Kraemer, GSI, D-64291 Darmstadt

   $Log$

*/
#include "dcmbasic.h"

#define DICOMDS2ATTRCASE(path,attr) \
      case DICOM##attr:\
         (path).psDE##attr = & (psDS->psDE[iDE]);\
      break;

#define DICOMDS2ATTRALLOC(path,stem)\
            if ( (\
            (path).ps##stem = calloc( (size_t) psDS->psDE[iDE].nDS, sizeof(*((path).ps##stem)) )\
                 ) == NULL ) {\
               sCE.iRC = ERRDICOM_SYS; goto gError;\
            }\
            (path).n##stem = psDS->psDE[iDE].nDS;


                                      /** \struct SDICOMSOPINSTANCEREFERENCE
                                          \brief 
                                          PS 3.3 
                                          Table 10-11 SOP Instance Reference Macro Description.
                                      */ 
struct SDICOMSOPINSTANCEREFERENCE {
   struct SDICOMDATAELEMENT 
          *psDEReferencedSOPClassUID
         ,*psDEReferencedSOPInstanceUID
         ;
};

#define DICOMSOPINSTANCEREFERENCEFROMDS(sir,ds)\
        {\
        int iDE; struct SDICOMDATASET *psDS = (ds);\
           for ( iDE=0; iDE<(ds)->nDE; iDE++ ) {\
              switch( (uint32_t) ((ds)->psDE[iDE].sDEH.ui16GN << 16) | (ds)->psDE[iDE].sDEH.ui16EN ) {\
              DICOMDS2ATTRCASE(*(sir),ReferencedSOPClassUID);\
              DICOMDS2ATTRCASE(*(sir),ReferencedSOPInstanceUID);\
              default: break;\
              }\
           }\
        }

#define DICOMSOPINSTANCEREFERENCEFROMDE(sir)\
              DICOMDS2ATTRCASE(*(sir),ReferencedSOPClassUID);\
              DICOMDS2ATTRCASE(*(sir),ReferencedSOPInstanceUID);

#define DICOMSOPINSTANCEREFERENCEINSPECT(sir)\
   fprintf( stderr, "<D> SDICOMSOPINSTANCEREFERENCE=%X\n", sir );\
   DICOMELEMENTDATAINSPECT(31,*sir,ReferencedSOPClassUID);\
   DICOMELEMENTDATAINSPECT(31,*sir,ReferencedSOPInstanceUID);

                                      /** \struct SDICOMIMAGESOPINSTANCEREFERENCE
                                          \brief 
                                          PS 3.3 
                                          Table 10-3 Image SOP Instance Reference Macro Description.
                                      */ 
struct SDICOMIMAGESOPINSTANCEREFERENCE {
   struct SDICOMSOPINSTANCEREFERENCE sSIR;
   struct SDICOMDATAELEMENT 
          *psDEReferencedFrameNumber
         ,*psDEReferencedSegmentNumber
         ;
};


/*
#define  DICOMDE2IMAGESOPINSTANCEREFERENCE(path,attr) \
      case DICOM##attr:\
         (path).psDE##attr = & psDE[iDE];\
      break;
*/

                                      /** \struct SDICOMCODESEQUENCE
                                          \brief 
                                          PS 3.3 
                                          Table 8.8-1 Common Attribute Set for Code Sequence Attributes.
                                      */ 
struct SDICOMCODESEQUENCE {
   struct SDICOMDATAELEMENT 
                                      /* basic coded entry attributes */
          *psDECodeValue
         ,*psDECodingSchemeDesignator
         ,*psDECodingSchemeVersion
         ,*psDECodeMeaning
                                      /* enhanced encoding mode */
         ,*psDEContextIdentifier
         ,*psDEMappingResource
         ,*psDEContextGroupVersion
         ,*psDEContextGroupExtensionFlag
         ,*psDEContextGroupLocalVersion
         ,*psDEContextGroupExtensionCreatorUID
         ;
};

/*

   Functions
   ---------
   
   dicomCodeSequenceFree   ( cs, flag )
                           o releases resources
                           o to test
                           o calls:
                            
   dicomCodeSequenceFromDS ( cs, ds, flag )
                           o fill CS from dataset
                           o to test
                           o calls:
   
   dicomCodeSequenceInit   ( cs, flag )
                           o initialize
                           o to test
                         
   dicomCodeSequenceInspect( cs, flag )
                           o inspect 
                           o to test
                           o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomCodeSequenceFree   ( struct SDICOMCODESEQUENCE *, unsigned int );
struct SCMDERR dicomCodeSequenceFromDS ( struct SDICOMCODESEQUENCE *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomCodeSequenceInit   ( struct SDICOMCODESEQUENCE *, unsigned int );
struct SCMDERR dicomCodeSequenceInspect( struct SDICOMCODESEQUENCE *, unsigned int );

struct SDICOMCODESEQUENCE _dicomCodeSequenceTest( unsigned int );


                                      /** \struct SDICOMPERSONID
                                          \brief 
                                          PS 3.3 
                                          Table 10-1 Person Identification Macro Attributes Description.
                                      */ 
struct SDICOMPERSONID {
   struct SDICOMDATAELEMENT 
          *psDEPersonsAddress
         ,*psDEPersonsTelephoneNumbers
         ,*psDEInstitutionName
         ,*psDEInstitutionAddress
         ;
                                      /* Person Identification Code Sequence */
   int nPIC;
   struct SDICOMCODESEQUENCE /*  Code Sequence Macro Table 8.8-1 */ *psPIC; 
                                      /* Institution Code Sequence */
   int nIC;
   struct SDICOMCODESEQUENCE /*  Code Sequence Macro Table 8.8-1 */ *psIC; 
};
/*

   Functions
   ---------
   
   dicomPersonIDFree    ( pi, flag )
                        o releases resources
                        o to test
                        o calls:
                            
   dicomPersonIDFromDS  ( pi, ds, flag )
                        o fill pi from dicom dataset
                        o to test
                        o calls:
                          o dicomCodeSequenceFromDS
   
   dicomPersonIDInit    ( pi, flag )
                        o initialize
                        o to test
                        o calls:
                         
   dicomPersonIDInspect ( pi, flag )
                        o inspect 
                        o to test
                        o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomPersonIDFree   ( struct SDICOMPERSONID *, unsigned int );
struct SCMDERR dicomPersonIDFromDS ( struct SDICOMPERSONID *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomPersonIDInit   ( struct SDICOMPERSONID *, unsigned int );
struct SCMDERR dicomPersonIDInspect( struct SDICOMPERSONID *, unsigned int );

struct SDICOMPERSONID _dicomPersonIDTest( unsigned int );


                                      /** \struct SDICOMCONTENTITEM
                                          \brief 
                                          PS 3.3 
                                          Table 10-2 Content Item Macro Attributes Description.
                                      */ 
struct SDICOMCONTENTITEM { 
   struct SDICOMDATAELEMENT 
          *psDEValueType
         ,*psDEDateTime
         ,*psDEDate
         ,*psDETime
         ,*psDEPersonName
         ,*psDEUID
         ,*psDETextValue
         ,*psDENumericValue
         ;
                                      /* Concept Name Code Sequence */
   int nCNC;
   struct SDICOMCODESEQUENCE *psCNC; 
                                      /* Concept Code Sequence */
   int nCC;
   struct SDICOMCODESEQUENCE *psCC; 
                                      /* Measurement Units Code Sequence */
   int nMUC;
   struct SDICOMCODESEQUENCE *psMUC; 
};

/*

   Functions
   ---------
   
   dicomContentItemFree   ( ci, flag )
                          o releases resources
                          o to test
                          o calls:
                            
   dicomContentItemFromDS ( ci, ds, flag )
                          o fill pi from dicom dataset
                          o to test
                          o calls:
                            o dicomCodeSequenceFromDS
   
   dicomContentItemInit   ( ci, flag )
                          o initialize
                          o to test
                          o calls:
                         
   dicomContentItemInspect( ci, flag )
                          o inspect 
                          o to test
                          o calls:
                         
*/

                                      /* function prototypes */
struct SCMDERR dicomContentItemFree   ( struct SDICOMCONTENTITEM *, unsigned int );
struct SCMDERR dicomContentItemFromDS ( struct SDICOMCONTENTITEM *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomContentItemInit   ( struct SDICOMCONTENTITEM *, unsigned int );
struct SCMDERR dicomContentItemInspect( struct SDICOMCONTENTITEM *, unsigned int );

struct SDICOMCONTENTITEM _dicomContentItemTest( unsigned int );

                                      /** \struct SDICOMPROTOCOLCONTEXT
                                          \brief 
                                          PS 3.3 
                                          Table 10-9 Request Attributes Macro Description.
                                      */ 
struct SDICOMPROTOCOLCONTEXT { 
   struct SDICOMCONTENTITEM sCI;
                                      /* Content Item Modifier Sequence */
   int nCIM;
   struct SDICOMCONTENTITEM *psCIM; 
};
/*

   Functions
   ---------
   
   dicomProtocolContextFree( pc, flag )
                        o releases resources
                        o to do
                        o calls:
                            
   dicomProtocolContextFromDS( pc, ds, flag )
                        o fill protocol context from dicom dataset
                        o to do
                        o calls:
   
   dicomProtocolContextInit    ( pc, flag )
                        o initialize
                        o to do
                        o calls:
                         
   dicomProtocolContextInspect ( pc, flag )
                        o inspect 
                        o to do
                        o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomProtocolContextFree   ( struct SDICOMPROTOCOLCONTEXT *, unsigned int );
struct SCMDERR dicomProtocolContextFromDS ( struct SDICOMPROTOCOLCONTEXT *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomProtocolContextInit   ( struct SDICOMPROTOCOLCONTEXT *, unsigned int );
struct SCMDERR dicomProtocolContextInspect( struct SDICOMPROTOCOLCONTEXT *, unsigned int );

struct SDICOMPROTOCOLCONTEXT _dicomProtocolContextTest( unsigned int );



                                      /** \struct SDICOMSCHEDULEDPROTOCOLCODE
                                          \brief 
                                          PS 3.3 
                                          Table 10-9 Request Attributes Macro Description.
                                      */ 
struct SDICOMSCHEDULEDPROTOCOLCODE {
   struct SDICOMCODESEQUENCE sCS;
   struct SDICOMDATAELEMENT 
          *psDERequestedProcedureID
         ;
                                      /*  */
   int nPC;
   struct SDICOMPROTOCOLCONTEXT *psPC; 
};
/*

   Functions
   ---------
   
   dicomScheduledProtocolCodeFree    ( spc, flag )
                        o releases resources
                        o to do
                        o calls:
                            
   dicomScheduledProtocolCodeFromDS  ( spc, ds, flag )
                        o fill pi from dicom dataset
                        o to do
                        o calls:
   
   dicomScheduledProtocolCodeInit    ( spc, flag )
                        o initialize
                        o to do
                        o calls:
                         
   dicomScheduledProtocolCodeInspect ( spc, flag )
                        o inspect 
                        o to do
                        o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomScheduledProtocolCodeFree   ( struct SDICOMSCHEDULEDPROTOCOLCODE *, unsigned int );
struct SCMDERR dicomScheduledProtocolCodeFromDS ( struct SDICOMSCHEDULEDPROTOCOLCODE *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomScheduledProtocolCodeInit   ( struct SDICOMSCHEDULEDPROTOCOLCODE *, unsigned int );
struct SCMDERR dicomScheduledProtocolCodeInspect( struct SDICOMSCHEDULEDPROTOCOLCODE *, unsigned int );

struct SDICOMSCHEDULEDPROTOCOLCODE _dicomScheduledProtocolCodeTest( unsigned int );



                                      /** \struct SDICOMREQUATTR
                                          \brief 
                                          PS 3.3 
                                          Table 10-9 Request Attributes Macro Description.
                                      */ 
struct SDICOMREQUATTR {
   struct SDICOMDATAELEMENT 
          *psDERequestedProcedureID
         ,*psDEAccessionNumber
         ,*psDEStudyInstanceUID
         ,*psDERequestedProcedureDescription
         ,*psDEReasonfortheRequestedProcedure
         ,*psDEScheduledProcedureStepID
         ,*psDEScheduledProcedureStepDescription
         ;
                                      /* Referenced Study Sequence */
   int nRStudy;
   struct SDICOMSOPINSTANCEREFERENCE *psRStudy; 
                                      /* Requested Procedure Code Sequence */
   int nRPC;
   struct SDICOMCODESEQUENCE *psRPC; 
                                      /* Reason for Requested Procedure Code Sequence */
   int nRRPC;
   struct SDICOMCODESEQUENCE *psRRPC; 
                                      /* Scheduled Protocol Code Sequence */
   int nSPC;
   struct SDICOMSCHEDULEDPROTOCOLCODE *psSPC; 
};
/*

   Functions
   ---------
   
   dicomRequAttrFree    ( pi, flag )
                        o releases resources
                        o to do
                        o calls:
                            
   dicomRequAttrFromDS  ( pi, ds, flag )
                        o fill pi from dicom dataset
                        o to do
                        o calls:
   
   dicomRequAttrInit    ( pi, flag )
                        o initialize
                        o to do
                        o calls:
                         
   dicomRequAttrInspect ( pi, flag )
                        o inspect 
                        o to do
                        o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomRequAttrFree   ( struct SDICOMREQUATTR *, unsigned int );
struct SCMDERR dicomRequAttrFromDS ( struct SDICOMREQUATTR *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomRequAttrInit   ( struct SDICOMREQUATTR *, unsigned int );
struct SCMDERR dicomRequAttrInspect( struct SDICOMREQUATTR *, unsigned int );

struct SDICOMREQUATTR _dicomRequAttrTest( unsigned int );

                                      /** \struct SDICOMPERFORMEDPROTOCOLCODE
                                          \brief 
                                          PS 3.3 
                                          Table 10-16.
                                      */ 
struct SDICOMPERFORMEDPROTOCOLCODE {
   struct SDICOMCODESEQUENCE sCS;
                                      /*  */
   int nPC;
   struct SDICOMPROTOCOLCONTEXT *psPC; 
};
                                      /** \struct SDICOMPERFORMEDPROCEDURESTEPSUMMARY
                                          \brief 
                                          PS 3.3 
                                          Table 10-16 performed procedure step summary macro attributes.
                                      */ 
struct SDICOMPERFORMEDPROCEDURESTEPSUMMARY {
   struct SDICOMDATAELEMENT 
          *psDEPerformedProcedureStepID
         ,*psDEPerformedProcedureStepStartDate
         ,*psDEPerformedProcedureStepStartTime
         ,*psDEPerformedProcedureStepDescription
         ,*psDECommentsonthePerformedProcedureStep
         ;
                                      /* Performed Protocol Code Sequence */
   int nPPC;
   struct SDICOMPERFORMEDPROTOCOLCODE *psPPC; 
};
/*

   Functions
   ---------
   
   dicomFree    ( pi, flag )
                        o releases resources
                        o to do
                        o calls:
                            
   dicomFromDS  ( pi, ds, flag )
                        o fill pi from dicom dataset
                        o to do
                        o calls:
                          o dicomCodeSequenceFromDS
   
   dicomInit    ( pi, flag )
                        o initialize
                        o to do
                        o calls:
                         
   dicomInspect ( pi, flag )
                        o inspect 
                        o to do
                        o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomFree   ( struct SDICOM *, unsigned int );
struct SCMDERR dicomFromDS ( struct SDICOM *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomInit   ( struct SDICOM *, unsigned int );
struct SCMDERR dicomInspect( struct SDICOM *, unsigned int );

struct SDICOM _dicomTest( unsigned int );

#if defined(LATER)
                                      /** \struct SDICOM
                                          \brief 
                                          PS 3.3 
                                          Table 10-x Macro Description.
                                      */ 
struct SDICOM {
   struct SDICOMDATAELEMENT 
          *psDE
         ,*psDE
         ;
                                      /*  */
   int n;
   struct SDICOM /*  */ *ps; 
};
/*

   Functions
   ---------
   
   dicomFree    ( pi, flag )
                        o releases resources
                        o to do
                        o calls:
                            
   dicomFromDS  ( pi, ds, flag )
                        o fill pi from dicom dataset
                        o to do
                        o calls:
                          o dicomCodeSequenceFromDS
   
   dicomInit    ( pi, flag )
                        o initialize
                        o to do
                        o calls:
                         
   dicomInspect ( pi, flag )
                        o inspect 
                        o to do
                        o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomFree   ( struct SDICOM *, unsigned int );
struct SCMDERR dicomFromDS ( struct SDICOM *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomInit   ( struct SDICOM *, unsigned int );
struct SCMDERR dicomInspect( struct SDICOM *, unsigned int );

struct SDICOM _dicomTest( unsigned int );
#endif


#endif
