#ifndef DCMSERIES_H
#define DCMSERIES_H
/** \file

   Definitions to handle DICOM Series Modules.

*/
/*

   Copyright  2005-2011 M.Kraemer, GSI, D-64291 Darmstadt

   $Log$

*/
#include "dcmbasic.h"
#include "dcmmacro.h"


                                      /** \struct SDICOMRELATEDSERIES
                                          \brief 
                                          C7.3.1 General Series Module, Table C.7-5a.
                                      */ 
struct SDICOMRELATEDSERIES {
   struct SDICOMDATAELEMENT 
          *psDEStudyInstanceUID
         ,*psDESeriesInstanceUID
         ;
                                      /* Purpose of Reference Code */
   size_t nPRC;
   struct SDICOMCODESEQUENCE *psPRC; 
};
                                      /** \struct SDICOMGENERALSERIESMODULE
                                          \brief 
                                          C7.3.1 General Series Module, Table C.7-5a.
                                      */ 
struct SDICOMGENERALSERIESMODULE {
   struct SDICOMDATAELEMENT 
          *psDEModality
         ,*psDESeriesInstanceUID
         ,*psDESeriesNumber
         ,*psDELaterality
         ,*psDESeriesDate
         ,*psDESeriesTime
         ,*psDEPerformingPhysiciansName
         ,*psDEProtocolName
         ,*psDESeriesDescription
         ,*psDEOperatorsName
         ,*psDEBodyPartExamined
         ,*psDEPatientPosition
         ,*psDESmallestPixelValueinSeries
         ,*psDELargestPixelValueinSeries
         ,*psDE/* Performed Procedure Step Summary Macro Table 10-16 */
         ;
   size_t nPPI;
   struct SDICOMPERSONID *psPPI; 
                                      /* Operator Identification */
   size_t nOI;
   struct SDICOMPERSONID *psOI; 
                                      /* Referenced Performed Procedure Step */
   size_t nRPPS;
   struct SDICOMSOPINSTANCEREFERENCE *psRPPS; 
                                      /* Related Series */
   size_t nRSeries;
   struct SDICOMRELATEDSERIES *psRSeries; 
                                      /* Request Attributes */
   size_t nRA;
   struct SDICOMREQUATTR *psRA; 
};
                                      /** \struct SDICOMCLINICALTRIALSERIESMODULE
                                          \brief
                                          C.7.3.2 Clinical Trial Series Module, Table C.7-5b.
                                      */ 
struct SDICOMCLINICALTRIALSERIESMODULE {
   struct SDICOMDATAELEMENT 
          *psDEClinicalTrialCoordinatingCenterName
         ,*psDEClinicalTrialSeriesID
         ,*psDEClinicalTrialSeriesDescription
         ;
};
                                      /** \struct SDICOMENHANCEDSERIESMODULE
                                          \brief
                                          C.7.3.3 Enhanced Series Module, Table C.7-5c.
                                      */ 
struct SDICOMENHANCEDSERIESMODULE {
   struct SDICOMDATAELEMENT 
          *psDESeriesNumber
         ;
                                      /* Referenced Performed Procedure Step */
   size_t nRPPS;
   struct SDICOMSOPINSTANCEREFERENCE *psRPPS; 
};
                                      /** \struct SDICOMSERIESMODULE
                                          \brief Holds Study Module Attributes.
                                      */ 
                                      /* PS 3.3
                                         C.7.3 Common Series IE Modules
                                      */
struct SDICOMSERIESMODULE {
   struct SDICOMGENERALSERIESMODULE sGSM;
   struct SDICOMCLINICALTRIALSERIESMODULE sCTSM;
   struct SDICOMENHANCEDSERIESMODULE sESM;
};
/*

   Functions
   ---------
   
   dicomSeriesModuleFree    ( rtss, flag )
                              o releases resources
                              o to do
                              o calls:
                            
   dicomSeriesModuleFromDS  ( rtss, dicomds, flag )
                              o build RT structure set from DICOM file
                              o to test
                              o calls:
   
   dicomSeriesModuleInit    ( rtss, flag )
                              o initialize
                              o to do
                              o calls:
                         
   dicomSeriesModuleInspect ( rtss, flag )
                              o inspect 
                              o to do
                              o calls:
                         
*/
                                      /* function prototypes */
struct SCMDERR dicomSeriesModuleFree   ( struct SDICOMSERIESMODULE *, unsigned int );
struct SCMDERR dicomSeriesModuleFromDS ( struct SDICOMSERIESMODULE *, struct SDICOMDATASET *, unsigned int );
struct SCMDERR dicomSeriesModuleInit   ( struct SDICOMSERIESMODULE *, unsigned int );
struct SCMDERR dicomSeriesModuleInspect( struct SDICOMSERIESMODULE *, unsigned int );

struct SDICOM _dicomSeriesModuleTest( unsigned int );

#endif
