#ifndef MTYPES_H 
#define MTYPES_H

/** \file Definitions for math datatypes.
*/
/*
   $Log: mtypes.h,v $
   Revision 1.1.1.1  2009/05/07 12:58:12  kraemer


   Revision 1.1.2.1  2009/05/06 22:26:29  kraemer
   *** empty log message ***

*/

#include <float.h>
#include <limits.h>
#include <math.h>

#include "osversion.h"

#ifndef min
#define min(a,b) ( (a) < (b) ) ? (a) : (b)
#endif

#ifndef max
#define max(a,b) ( (a) > (b) ) ? (a) : (b)
#endif
 
#if defined(AMIGA)

 
#if defined(M881)

#include <m68881.h>
#undef  floor
#define floor floor

#elif defined(MFFP)
#elif defined(MIEEE)
#else
#error You loose, choose an appropriate floating point option ! 
#endif             /* end M881/MFFP/MIEEE */

 

 
#if defined(MFFP)
 
#undef DBL_MAX
#undef FLT_MAX
#undef DBL_MIN
#undef FLT_MIN
#define DBL_MAX 9.2233717E18
#define FLT_MAX 9.2233717E18
/*#define DBL_MIN 2.7105E-20 */
/*#define FLT_MIN 2.7105E-20 */
 
union UNDDUL { unsigned long ul; double dd; };

static union UNDDUL uDBL_MIN = { 0x80000000 };
#undef DBL_MIN
#define DBL_MIN (uDBL_MIN.dd)
#define FLT_MIN DBL_MIN

static union UNDDUL uDBL_EPSILON = { 0x8000002A };
#undef DBL_EPSILON
#define DBL_EPSILON (uDBL_EPSILON.dd)
#undef FLT_EPSILON
#define FLT_EPSILON DBL_EPSILON
 
static union UNDDUL uDBL_QNAN = { 0x0000007F };
#define DBL_QNAN (uDBL_QNAN.dd)
 
  
union UNRRUL { unsigned long ul; float rr; };
static union UNRRUL uFLT_QNAN = { 0x0000007F };
#define FLT_QNAN (uFLT_QNAN.rr)

static union UNDDUL uDBL_INF = { 0x0000007F };
#define DBL_INF (uDBL_INF.dd)

static union UNRRUL uFLT_INF = { 0x0000007F };
#define FLT_INF (uFLT_INF.rr)

#define FLT_SNAN FLT_QNAN
#define DBL_SNAN DBL_QNAN


#define ISDBLSKIP(x) ( ((x)==DBL_QNAN) )
#define ISFLTSKIP(x) ( ((x)==FLT_QNAN) )
 
  
#else
  
  
union UNDDUL { unsigned long ulv[2]; double dd; };

#ifndef DBL_QNAN
#ifndef _QNAN
static union UNDDUL uDBL_QNAN = { 0x7FFFFFFF, 0xFFFFFFFF };
#define DBL_QNAN (uDBL_QNAN.dd)
#else
#define DBL_QNAN _QNAN
#endif
#endif

#ifndef DBL_SNAN
#ifndef _SNAN
static union UNDDUL uDBL_SNAN = { 0x7FF7FFFF, 0xFFFFFFFF };
#define DBL_SNAN (uDBL_SNAN.dd)
#else
#define DBL_SNAN _SNAN
#endif
#endif
 
union UNRRUL { unsigned long ul; float rr; };

#ifndef FLT_QNAN
#ifndef _QNAN
static union UNRRUL uFLT_QNAN = { 0x7FFFFFFF };
#define FLT_QNAN (uFLT_QNAN.rr)
#else
#define FLT_QNAN _QNAN
#endif
#endif

#ifndef FLT_SNAN
#ifndef _SNAN
static union UNRRUL uFLT_SNAN = { 0x7FBFFFFF };
#define FLT_SNAN (uFLT_SNAN.rr)
#else
#define FLT_SNAN _SNAN
#endif
#endif

static union UNDDUL uDBL_INF = { 0x7FF00000, 0x00000000 };
#define DBL_INF (uDBL_INF.dd)
static union UNRRUL uFLT_INF = { 0x7F800000 };
#define FLT_INF (uFLT_INF.rr)


int isdblskip(double);
#define ISDBLSKIP(x) isdblskip(x)
int isfltskip(float);
#define ISFLTSKIP(x) isfltskip(x)

#endif



#ifndef isfinite
#define isfinite(x) \
    ( (sizeof(x) == sizeof(float)  ) ? isfiniter(x) \
   :  (sizeof(x) == sizeof(double) ) ? isfinited(x) \
   :                                   isfiniteq(x))
int isfiniter( float );
int isfinited( double );    
int isfiniteq( double );    
#endif

#ifndef M_PI
#define M_PI PI
#endif
#ifndef M_PI_2
#define M_PI_2 PID2
#endif
#ifndef M_PI_4
#define M_PI_4 PID4
#endif
#ifndef M_2PI
#define M_2PI (PI+PI)
#endif
#ifndef M_LN2
#define M_LN2 log(2.0E0)
#endif

#ifndef M_E
#define M_E 2.71828182845904523536028747135266250 
#endif

                                      /* these exist */
#define POW2_EXISTS
#define COT_EXISTS
 

#elif defined(_AIX) || defined(__hpux) || defined(sun) || defined(__mips) || \
      defined(__linux__) || defined(__osf__)
 
 
 
union UNDDUL { unsigned long long ull; double dd; };

#ifndef DBL_QNAN
#ifndef _QNAN
static union UNDDUL uDBL_QNAN = { 0x7FF8000000000000ULL };
#define DBL_QNAN (uDBL_QNAN.dd)
#else
#define DBL_QNAN _QNAN
#endif
#endif

#ifndef FLT_QNAN
static unsigned int rQNAN = 0x7FC00000;
#define FLT_QNAN   (*((float *) (&rQNAN)))
#endif
 
union UNRRUL { unsigned long ul; float rr; };

/*#define ISDBLSKIP(x) ( ((x)==DBL_QNAN) ) */
/*#define ISFLTSKIP(x) ( ((x)==FLT_QNAN) ) */
#define ISDBLSKIP(x) ( isnan(x) )
#define ISFLTSKIP(x) ( isnan(x) )

#ifndef isfinited
#define isfinited finite
#endif
 
#define PI M_PI
#ifndef M_2PI
#define M_2PI (PI+PI)
#endif
 
#define ASINH_EXISTS
#define ACOSH_EXISTS
#define ATANH_EXISTS 

#elif defined(__EMX__)  || defined(__APPLE_CC__) 
 
 
union UNDDUL { unsigned long long ull; double dd; };

#ifndef DBL_QNAN
#ifndef _QNAN
static union UNDDUL uDBL_QNAN = { 0x7FF8000000000000ULL };
#define DBL_QNAN (uDBL_QNAN.dd)
#else
#define DBL_QNAN _QNAN
#endif
#endif
 
union UNRRUL { unsigned long ul; float rr; };
#ifndef FLT_QNAN
#ifndef _QNAN
static union UNRRUL uFLT_QNAN = { 0x7FC00000};
#define FLT_QNAN (uFLT_QNAN.rr)
#else
#define FLT_QNAN _QNAN
#endif
#endif

#define ISDBLSKIP(x) ( __isnan(x) )
#define ISFLTSKIP(x) ( __isnanf(x) )
#define isinf(x) ( !isfinite(x) )
 
#define PI M_PI
#ifndef M_2PI
#define M_2PI (PI+PI)
#endif
 
 
#elif ( defined(MVS) || defined(mvs) )
 
 
#define M_PI 3.141592653589793238462643E0
#define PI M_PI
#define min(a,b) ( (a) < (b) ) ? (a) : (b)
#define max(a,b) ( (a) > (b) ) ? (a) : (b)
 
#else
#error You loose, you should not use this computer
#endif
/*
#define DBL_SKIP -DBL_MAX
#define FLT_SKIP -FLT_MAX
*/
#define DBL_SKIP DBL_QNAN
#define FLT_SKIP FLT_QNAN

/*
   --- special function macros
*/
#ifndef POW2_EXISTS
#define  pow2(x) pow(2.0E0,x)
#endif

#define pow10(x) pow(1.0E1,x)

#ifndef LOG2_EXISTS
#define log2(x) ( log(x) / M_LN2 )           
#endif

#ifndef COT_EXISTS
#define  cot(x) 1E0/tan(x)
#endif

#define SIND(deg) (\
                  ( fmod(deg,180.) ==   0 ) ? (0.0E0) : \
                  ( fmod(deg-90.,360.) == 0 ) ? (1E0) : \
                  ( fmod(deg+90.,360.) == 0 ) ? (-1E0) : \
                  ( fmod(deg,360.) ==  30 ) ? (0.5) : \
                  ( fmod(deg,360.) == 150 ) ? (0.5) : \
                  ( fmod(deg,360.) ==  45 ) ? sqrt(0.5) : \
                  ( fmod(deg,360.) == 135 ) ? sqrt(0.5) : \
                  ( fmod(deg,360.) ==  60 ) ? sqrt(0.75) : \
                  ( fmod(deg,360.) == 120 ) ? sqrt(0.75) : \
                  ( fmod(deg,360.) == 210 ) ? (-0.5) : \
                  ( fmod(deg,360.) == 330 ) ? (-0.5) : \
                  ( fmod(deg,360.) == 225 ) ? -sqrt(0.5) : \
                  ( fmod(deg,360.) == 315 ) ? -sqrt(0.5) : \
                  ( fmod(deg,360.) == 240 ) ? -sqrt(0.75) : \
                  ( fmod(deg,360.) == 300 ) ? -sqrt(0.75) : \
                  ( fmod(deg,360.) == -30 ) ? (-0.5) : \
                  ( fmod(deg,360.) ==-150 ) ? (-0.5) : \
                  ( fmod(deg,360.) == -45 ) ? -sqrt(0.5) : \
                  ( fmod(deg,360.) ==-135 ) ? -sqrt(0.5) : \
                  ( fmod(deg,360.) == -60 ) ? -sqrt(0.75) : \
                  ( fmod(deg,360.) ==-120 ) ? -sqrt(0.75) : \
                  ( fmod(deg,360.) ==-210 ) ? (0.5) : \
                  ( fmod(deg,360.) ==-330 ) ? (0.5) : \
                  ( fmod(deg,360.) ==-225 ) ? sqrt(0.5) : \
                  ( fmod(deg,360.) ==-315 ) ? sqrt(0.5) : \
                  ( fmod(deg,360.) ==-240 ) ? sqrt(0.75) : \
                  ( fmod(deg,360.) ==-300 ) ? sqrt(0.75) : \
                  sin( (deg) * PI / 180E0 ) ) 

#define COSD(deg) (\
                  ( fmod(deg,360.) ==   0 ) ? (1.0E0) : \
                  ( fmod(deg-180.,360.) ==   0 ) ? (-1.0E0) : \
                  ( fmod(deg-90.,180.) == 0 ) ? (0E0) : \
                  ( fmod(deg,360.) ==  60 ) ? (0.5) : \
                  ( fmod(deg,360.) == 300 ) ? (0.5) : \
                  ( fmod(deg,360.) ==  45 ) ? sqrt(0.5) : \
                  ( fmod(deg,360.) == 315 ) ? sqrt(0.5) : \
                  ( fmod(deg,360.) ==  30 ) ? sqrt(0.75) : \
                  ( fmod(deg,360.) == 330 ) ? sqrt(0.75) : \
                  ( fmod(deg,360.) == 120 ) ? (-0.5) : \
                  ( fmod(deg,360.) == 240 ) ? (-0.5) : \
                  ( fmod(deg,360.) == 135 ) ? -sqrt(0.5) : \
                  ( fmod(deg,360.) == 225 ) ? -sqrt(0.5) : \
                  ( fmod(deg,360.) == 150 ) ? -sqrt(0.75) : \
                  ( fmod(deg,360.) == 210 ) ? -sqrt(0.75) : \
                  ( fmod(deg,360.) ==-120 ) ? (-0.5) : \
                  ( fmod(deg,360.) ==-240 ) ? (-0.5) : \
                  ( fmod(deg,360.) ==-225 ) ? -sqrt(0.5) : \
                  ( fmod(deg,360.) ==-135 ) ? -sqrt(0.5) : \
                  ( fmod(deg,360.) ==-150 ) ? -sqrt(0.75) : \
                  ( fmod(deg,360.) ==-210 ) ? -sqrt(0.75) : \
                  ( fmod(deg,360.) == -60 ) ? (0.5) : \
                  ( fmod(deg,360.) ==-300 ) ? (0.5) : \
                  ( fmod(deg,360.) == -45 ) ? sqrt(0.5) : \
                  ( fmod(deg,360.) ==-315 ) ? sqrt(0.5) : \
                  ( fmod(deg,360.) == -30 ) ? sqrt(0.75) : \
                  ( fmod(deg,360.) ==-330 ) ? sqrt(0.75) : \
                  cos( (deg) * PI / 180E0 ) )

#define TAND(deg) (\
                  ( fmod(deg,180.) == 0 ) ? (0) : \
                  ( fmod(deg-90.,180.) == 0 ) ? (DBL_SKIP) : \
                  ( fmod(deg-45.,180.) == 0 ) ? 1.0 : \
                  ( fmod(deg+45.,180.) == 0 ) ? -1.0 : \
                  ( fmod(deg-30.,180.) == 0 ) ? sqrt(3.0)/3.0 : \
                  ( fmod(deg+30.,180.) == 0 ) ? -sqrt(3.0)/3.0 : \
                  ( fmod(deg-60.,180.) == 0 ) ? sqrt(3.0) : \
                  ( fmod(deg+60.,180.) == 0 ) ? -sqrt(3.0) : \
                  tan( (deg) * PI / 180E0 ) )

#define COTD(deg) (\
                  ( fmod(deg,180.) == 0 ) ? (DBL_SKIP) : \
                  ( fmod(deg-90.,180.) == 0 ) ? (0) : \
                  ( fmod(deg-45.,180.) == 0 ) ? 1.0 : \
                  ( fmod(deg+45.,180.) == 0 ) ? -1.0 : \
                  ( fmod(deg-30.,180.) == 0 ) ? sqrt(3.0) : \
                  ( fmod(deg+30.,180.) == 0 ) ? -sqrt(3.0) : \
                  ( fmod(deg-60.,180.) == 0 ) ? sqrt(3.0)/3. : \
                  ( fmod(deg+60.,180.) == 0 ) ? -sqrt(3.0)/3. : \
                  cot( (deg) * PI / 180E0 ) )

#define ASIND(arg) (\
                  ( (arg) ==   0 ) ? (0.0E0) : \
                  ( (arg) == 0.5 ) ? (30) : \
                  ( (arg) == 1.0E0 ) ? (90) : \
                  ( (arg) == -0.5 ) ? (-30) : \
                  ( (arg) == -1.0E0 ) ? (-90) : \
                  asin( (arg) ) * 180E0 / PI ) 

#define ACOSD(arg) (\
                  ( (arg) ==   0 ) ? (90) : \
                  ( (arg) == 0.5 ) ? (60) : \
                  ( (arg) == 1.0E0 ) ? (0) : \
                  ( (arg) == -0.5 ) ? (120) : \
                  ( (arg) == -1.0E0 ) ? (180) : \
                  acos( (arg) ) * 180E0 / PI ) 

#define ATAND(arg) (\
                  ( (arg) ==   0 ) ? (0) : \
                  atan( (arg) ) * 180E0 / PI )

#define ATAN2D(y,x) (\
                  ( ( (y) == 0E0 ) && ( (x) == (0.0E0) ) ) ? (DBL_QNAN) : \
                  ( ( (y)  > 0E0 ) && ( (x) == (0.0E0) ) ) ? (90) : \
                  ( ( (y)  < 0E0 ) && ( (x) == (0.0E0) ) ) ? (-90) : \
                  ( ( (y) == 0E0 ) && ( (x)  > (0.0E0) ) ) ? (  0) : \
                  ( ( (y) == 0E0 ) && ( (x)  < (0.0E0) ) ) ? (180) : \
                  ( ( (y)  > 0E0 ) && ( (x) == (y) ) ) ? (45) : \
                  ( ( (y)  > 0E0 ) && ( (x) == -(y) ) ) ? (135) : \
                  ( ( (y)  < 0E0 ) && ( (x) == (y) ) ) ? (-135) : \
                  ( ( (y)  < 0E0 ) && ( (x) == -(y) ) ) ? (-45) : \
                  atan2( y, x ) * 180E0 / PI )

#ifndef ASINH_EXISTS
#define asinh(x) log((x)+sqrt((x)*(x)+1.0E0))
#endif
#ifndef ACOSH_EXISTS
#define acosh(x) log((x)+sqrt((x)*(x)-1.0E0))
#endif
#ifndef ATANH_EXISTS
#define atanh(x) log((1.0E0+(x))/(1.0E0-(x)))*0.5
#endif


/*
   --- action flags and constants
*/
                                   /* for (vector) comparisons */
enum { GT=1, LT, NE, EQ, GE, LE, };
                                      /* */
                                      /* common error handling for vector functions... */
                                      /* error flag bits (lower bits keep array index 0..nn-1) */
#if defined(OLD)
#define ERRVMATH_ERROR   0xC0000000
#define ERRVMATH_ERRMASK 0xC0000000
#define ERRVMATH_SYS     0x80000000  
#define ERRVMATH_BADADDR 0x40000000  
#define ERRVMATH_DOMAIN  0x80000000  /* domain error ( sqrt(<0) etc ) */ 
#define ERRVMATH_ZDVD    0x40000000  /* zero-divide */
#endif

#define ERRVMATH_SYS     ( (unsigned long) LONG_MIN )
#define ERRVMATH_BADADDR ( (unsigned long) LONG_MIN >> 1 )
#define ERRVMATH_DOMAIN  ( (unsigned long) LONG_MIN )
#define ERRVMATH_ZDVD    ( (unsigned long) LONG_MIN >> 1 )
#define ERRVMATH_ERRMASK ( ERRVMATH_SYS | ERRVMATH_BADADDR )

#endif
