#if !defined(M_POLFIT_H)
#define M_POLFIT_H

/** \file Definitions for polynomial fits.
*/
/*
   $Log: polfit.h,v $
   Revision 1.1.1.1  2009/05/07 12:58:13  kraemer


   Revision 1.1.2.2  2009/05/07 12:36:51  kraemer
   64bit-clean

   Revision 1.1.2.1  2005/03/15 14:26:06  kraemer
   *** empty log message ***

*/


                                      /** \struct SPOLFITPAR
                                          \brief Holds polynomial fit parameters.
                                      */ 
struct SPOLFITPAR {
   unsigned int uiFlag;               /**< POLFIT_NOERR,... */
   int np;                            /**< inp: # of parameters */
   ssize_t nDF;                       /**< out: # degrees of freedom */
   double dChisq,                     /**< out: chisquare */
          dProb,                      /**< out: probability */
          dConf;                      /**< inp: (0..1) confidence limit */
   double *pdPar, *pdErr;             /**< out: [np] parameters, errors */
   double *pdCovar;                   /**< out: [np*np] covariance matrix */
   ssize_t *plCol, *plRow;            /**< out: [np] aux. for matrix inversion */
};

int polfitParAlloc  ( int, struct SPOLFITPAR *, unsigned int );
int polfitParDup    (      struct SPOLFITPAR *, struct SPOLFITPAR *, unsigned int );
int polfitParFree   (      struct SPOLFITPAR *, unsigned int );
int polfitParInit   (      struct SPOLFITPAR *, unsigned int );
int polfitParInspect(      struct SPOLFITPAR *, unsigned int );
int polfitParIsValid(      struct SPOLFITPAR *, unsigned int );

struct SPOLFITPAR *_polfitParTest( unsigned int );

                                      /** \struct SPOLFITDATA
                                          \brief Holds fit data.
                                      */ 
struct SPOLFITDATA {
   unsigned int uiFlag;
   size_t nn;                         /**< inp: # of data points */
   double *pdW;                       /**< scratch: weights */
   double *pdX, *pddXm, *pddXp;       /**< inp: x-values and -errors */
   double *pdY, *pddYm, *pddYp;       /**< inp: y-values and -errors */
   double *pdF, *pddFm, *pddFp;       /**< out: function values and -errors */
};

int polfitDataAlloc  ( size_t, struct SPOLFITDATA *, unsigned int );
int polfitDataFree   (         struct SPOLFITDATA *, unsigned int );
int polfitDataDup    (         struct SPOLFITDATA *, struct SPOLFITDATA *, unsigned int );
int polfitDataInspect(         struct SPOLFITDATA *, unsigned int );

int polfitFit( struct SPOLFITDATA *, struct SPOLFITPAR *, unsigned int );
int polfitFct( struct SPOLFITDATA *, struct SPOLFITPAR *, unsigned int );

                                      /* flags */
#define POLFIT_DEBUG 0x80000000       /* global debug */
#define POLFIT_NOERR 0x00010000       /* unknown data errors on input */

                                      /* error handling */
#define POLFIT_ERROR     0x80000000
#define POLFIT_WARNING   0x40000000
#define POLFIT_ERRORMASK 0xF0000000

int polfitStrError( int, char *, size_t  );

enum {
   ERRPOLFIT_SYS=1,
   ERRPOLFIT_BADADDR,
   ERRPOLFIT_BADPARCOUNT,
   ERRPOLFIT_BADDATACOUNT,
   ERRPOLFIT_NDF,
   ERRPOLFIT_INVMAT,
};
/*
int   vlsqpol( double *, double *, double *, double *, int,
               double *, double *, double *, int, double *
             );
*/ 

#endif
