#ifndef TRX_H
#define TRX_H
/** \file
   All TRAX definitions.
*/
/*

   Copyright  1992-2010 M.Kraemer, GSI Darmstadt

   $Log: trx.h,v $
   Revision 1.6  2009/05/29 13:11:03  kraemer
   next version; 64-bit clean; reworked projectiles/targets

   Revision 1.5  2008/06/20 14:22:37  kraemer
   x98 libs

   Revision 1.4  2008/02/05 10:55:32  kraemer
   doxygenized

   Revision 1.3  2002/11/25 16:09:28  kraemer
   added new command prototypes

   Revision 1.2  2001/01/15 17:23:36  kraemer
   *** empty log message ***

   Revision 1.1  2001/01/09 14:54:27  kraemer
   *** empty log message ***


*/

#include "u.h"
#include "m.h"

#include "trxtarg.h"
#include "trxxs.h"
#include "trxrnd.h"
#include "trxgeo.h"
#include "trxsrc.h"
#include "trxlmd.h"
#include "trxrun.h"
#if defined(LATER)
#endif
                                      /** \struct STRX
                                          \brief Main structure.
                                      */ 
struct STRX {
   unsigned int uiFlag;              /**< global system status flag */
   struct SCMD *psCmd;                /**< command subsystem reference */
   struct STRXPROJBASE sPB;           /**< projectile base */     
   struct STRXTARGBASE sTB;           /**< target base */     
   struct STRXXSBASE sXSB;            /**< cross section base */     
   struct STRXRND sRnd;               /**< random numbers */
   struct STRXGEO sGeo;               /**< geometry */
   struct STRXSRC sSrc;               /**< source */
   struct STRXLMD sLmd;               /**< list mode data */
   struct STRXRUN sRun;               /**< run data */
#if defined(LATER)
#endif   
};
                                      /* */
/*
   --- Common status and action flags ----------------------------------------
*/
                                      /* */
                                      /* masks */
#define TRX_MASKGLOBAL 0xFF000000     /* global (e.g. debug) */
#define TRX_MASKFUNC   0x00FF0000     /* function specific flags */
#define TRX_MASKQUAL   0x0000FFFF     /* for additional qualifiers */
                                      /* flags */
#define TRX_DEBUG      0x80000000     /* general check flag */
#define TRX_CHECK      TRX_DEBUG      /* general check flag */
#define TRX_QUIET      0x10000000     /* suppress error messages */
#define TRX_NOMSG      TRX_QUIET

/*
   --- function prototypes
*/
int TRXError( struct SCMDERR *, unsigned int );
int TRXInit ( struct STRX *, struct SCMD *, unsigned int );
int TRXTerm ( struct STRX *, unsigned int );

int TRXCmdRnd(
               int32_t *, int32_t *,
               int32_t *, int32_t *,
               const char *, const char *,
               int32_t *,
               int32_t *,
               int32_t *,
               int32_t *
             );
 
int TRXCmdGeo(
              const char *,
              const char *, const char *, const char *,
              int32_t *, int32_t *, int32_t *, int32_t *, int32_t *, int32_t *,  
              int32_t *  
             );

int TRXCmdSrc(
              const char *,
              const char *, const char *, const char *,
              int32_t *, int32_t *, int32_t *, int32_t *, int32_t *, int32_t *,  
              double *, const char *, const char *,    
              int32_t *  
             );

int TRXCmdLmd( const char *, int32_t *, int32_t *, const char *, const char *,
               int32_t *, 
               int32_t * );

int TRXCmdRun( int32_t *, const char *, int32_t *, int32_t * );

int TRXCmdScore( int32_t * );

                                      /*  */
int TRXCmdEqui( const char *, const char *, double *,
                int32_t * );
            
int TRXCmdEMFldTest   ( const char *, int32_t * );
int TRXCmdRndTest     ( const char *, int32_t * );
int TRXCmdVolTest     ( const char *, int32_t * );
int TRXCmdGeoTest     ( const char *, int32_t * );
int TRXCmdParticleTest( const char *, int32_t * );
int TRXCmdSrcTest     ( const char *, int32_t * );
int TRXCmdRunTest     ( const char *, int32_t * );
int TRXCmdLmdTest     ( const char *, int32_t * );

int TRXCmdTest        ( const char *, int32_t * );

/*
   --- error handling
*/
enum { 
   ERRTRX_SYS=1,
   ERRTRX_BADADDR,
   ERRTRX_CMDFAILED,
   ERRTRX_CMDPARNOTYET,
   ERRTRX_NOFILE,
   ERRTRX_BADNUMBER,
   ERRTRX_NOXSFORMATERIAL,
   ERRTRX_FILEEMPTY,
   ERRTRX_LOADFAILED,
   ERRTRX_BINDFAILED,
   ERRTRX_INITFAILED,
   ERRTRX_NOSUCHTARGET
};

#endif
