#ifndef TRXEMFLD_H
#define TRXEMFLD_H
/*
   
   Definitions for electromagnetic field

   Copyright  2002 M.Kraemer, GSI Darmstadt 

   $Log: trxemfld.h,v $
   Revision 1.1  2002/11/25 18:42:32  kraemer
   *** empty log message ***


*/

#include "cmderr.h"

/*
   --- Common status and action flags ----------------------------------------
*/
                                      /* */
                                      /* masks */
#define TRXEMFLD_MASKGLOBAL 0xFF000000 /* global (e.g. debug) */
#define TRXEMFLD_MASKFUNC   0x00FF0000 /* function specific flags */
#define TRXEMFLD_MASKQUAL   0x0000FFFF /* for additional qualifiers */

                                      /* flags */
#define TRXEMFLD_DEBUG      0x80000000 /* general check flag */
#define TRXEMFLD_CHECK      TRXEMFLD_DEBUG/* general check flag */
#define TRXEMFLD_QUIET      0x10000000 /* suppress error messages */
#define TRXEMFLD_NOMSG      TRXEMFLD_QUIET


                                      /* structure describing EM field */
struct STRXEMFLD {
   unsigned int uiFlag;              /* status bits */
   unsigned int uiType;              /* electric/magnetic */
   size_t ulSpace;             /*  */
   double dvHomo[3];                  /* constant field components 
                                         E: [V/cm]
					 B: [T] 
                                      */
   struct SCMDERR sCE;
};
                                      /* status bits */
#define TRXEMFLD_ACTIVE 0x000000001   /* any field active */
#define TRXEMFLD_HOMO   0x000000002   /* EM-field is homogeneous */

enum { TRXEMFLD_E=1, TRXEMFLD_M=2, };
                                      /* */
#define TRXEMFLD_FILEVERSION 20020124
#define TRXEMFLD_FILETYPE    "trxemfld"

/*

   TRXEMFldFree      : ( emfld, flag ) 
                     3 x to test
                       o calls
                         o TRXEMFldInit()
                
   TRXEMFldInit      : ( emfld, flag ) 
                     2 x to test
                       o calls
                
   TRXEMFldInspect   : ( emfld, flag ) 
                     1 x to test
                       o calls
                
   TRXEMFldIsValid   : ( emfld, flag ) 
                     4 x to test
                       o calls
                
   TRXEMFldSpace     : ( emfld, flag ) 
                     6 x to test
                       o calls
                
   TRXEMFld2File     : ( emfld, fp, flag ) 
                     5 x to test
                       o calls
                

*/
                                      /* function prototypes */
int TRXEMFldFree    ( struct STRXEMFLD *, unsigned int );
int TRXEMFldInit    ( struct STRXEMFLD *, unsigned int );
int TRXEMFldInspect ( struct STRXEMFLD *, unsigned int );
int TRXEMFldIsValid ( struct STRXEMFLD *, unsigned int );
int TRXEMFldSpace   ( struct STRXEMFLD *, unsigned int );
int TRXEMFld2File   ( struct STRXEMFLD *, FILE *, unsigned int );

struct STRXEMFLD *_TRXEMFldTestE( unsigned int );
struct STRXEMFLD *_TRXEMFldTestM( unsigned int );
                                      /* error handling */
int TRXEMFldError( struct SCMDERR *, unsigned int );

enum { ERRTRXEMFLD_SYS=1,
       ERRTRXEMFLD_BADADDR,
};
                                      /* structure to evaluate field */
struct STRXEMFLDEVAL {
   unsigned int uiFlag;              /* status bits */
   unsigned int uiType;              /* */
   double dE, dB;                     /* absolute magnitude */
   double dvx[3];                     /* location */
   double dvF[4][4];                  /*
                                         field strength tensor 
					 
					  0     Bz*c -By*c Ex
					 -Bz*c  0     Bx*c Ey
					  By*c -Bx*c  0    Ez
					  0     0     0    1
					  
					  E   [V/cm]
					  B*c [V/cm] = B[Vsec/m**2] * c[m/sec] 
                                                     = B[T] * c[m/sec] * 1E-2 
                                      */
   double dvFGrad[3][4][4];           /* field strength gradient */
   struct SCMDERR sCE;
};
/*

   TRXEMFldEval        ( emfldeval, emfld, flag ) 
                     4 x to test
		       o field strength tensor
                       o calls
                         
   TRXEMFldEvalFree    ( emfldeval, flag ) 
                     3 x to test
                       o calls
                         
   TRXEMFldEvalInit    ( emfldeval, flag ) 
                     2 x to test
                       o calls
                         
   TRXEMFldEvalInspect ( emfldeval, flag ) 
                     1 x to test
                       o calls                        
                
*/

#define TRXEMFLD_EVALMAGNITUDE 0x00010000
#define TRXEMFLD_EVALGRADIENT  0x00020000

                                      /* function prototypes */
int TRXEMFldEval        ( struct STRXEMFLDEVAL *, struct STRXEMFLD *, unsigned int );
int TRXEMFldEvalFree    ( struct STRXEMFLDEVAL *, unsigned int );
int TRXEMFldEvalInit    ( struct STRXEMFLDEVAL *, unsigned int );
int TRXEMFldEvalInspect ( struct STRXEMFLDEVAL *, unsigned int );

struct STRXEMFLDEVAL *_TRXEMFldEvalTest( unsigned int );

#endif
