#ifndef TRXEVAL_H
#define TRXEVAL_H
/** \file
   Common definitions for evaluation functions.
*/
/*
  
   Copyright  1992-2010 M.Kraemer, GSI Darmstadt 

   $Log: trxeval.h,v $
   Revision 1.6  2009/05/29 13:06:16  kraemer
   reworked projectiles, targets

   Revision 1.5  2008/06/20 14:24:46  kraemer
   internal rework eval, user evaluation function

   Revision 1.4  2008/02/05 11:19:52  kraemer
   eval rework

   Revision 1.3  2002/11/25 16:20:49  kraemer
   volmode, xsmax

   Revision 1.2  2001/01/15 16:52:30  kraemer
   bug fixes

   Revision 1.1  2001/01/09 14:54:30  kraemer
   *** empty log message ***


*/

#include "cmd.h"
#include "trxproj.h"
#include "trxxs.h"
#include "trxvol.h"
#include "trxparticle.h"

                                      /* masks */
#define TRXEVAL_MASKGLOBAL 0xFF000000 /* global (e.g. debug) */
#define TRXEVAL_MASKFUNC   0x00FF0000 /* function specific flags */
#define TRXEVAL_MASKQUAL   0x0000FFFF /* for additional qualifiers */

                                      /* flags */
#define TRXEVAL_DEBUG      0x80000000 /* general check flag */
#define TRXEVAL_CHECK      TRXEVAL_DEBUG  /* general check flag */
#define TRXEVAL_QUIET      0x10000000 /* suppress error messages */
#define TRXEVAL_NOMSG      TRXEVAL_QUIET
                                      /* */
                                      /** \struct STRXEVALLINETRACKSEQ
                                          \brief line track sequence.
                                      */ 
struct STRXEVALLINETRACKSEQ {
   double dvx[4];
   float rdE;
   float rE;
   unsigned int uiPT;
            int  iIA;
};    
                                      /** \struct STRXEVALLINETRACKID
                                          \brief line track id.
                                      */ 
struct STRXEVALLINETRACKID {
   unsigned int uiSq;                /* # of interactions */ 
   struct STRXEVALLINETRACKSEQ *psSq; /* [ uiSq ] */ 
};    
                                      /** \struct STRXEVALLINETRACKTRK
                                          \brief a track.
                                      */ 
struct STRXEVALLINETRACKTRK {
   unsigned int uiID;                 /* # of individual particles */ 
   struct STRXEVALLINETRACKID *psID;  /* [uiID] */ 
};    
                                      /** \struct STRXEVALLINETRACK
                                          \brief line track structure.
                                      */ 
struct STRXEVALLINETRACK {
   double dvLimz[2];                  /**< axial limits [cm] */
   struct STRXEVALLINETRACKTRK *psT;  /**< [ulEvt] tracks */         
};
                                      /** \struct STRXEVALADOSE
                                          \brief For evaluation of axial dose.
                                      */ 
struct STRXEVALADOSE {
   unsigned int uizScale;             /**< lin/log/... */
   int nz;                            /**< # of axial bins */
   double *pdzBin;                    /**< [nz+1] axial bins [cm] */
   double *pdzDoseE;                  /**< [nz] axial energy deposition [keV] */
   unsigned int *puizDoseI;           /**< [nz] axial ionization deposition */
   unsigned int *puizCountE;          /**< [nz] axial energy deposition [counts] */
   double dvLimr[2];                  /**< radial limits [cm] */
   double dvLimz[2];                  /**< axial limits [cm] */
};
                                      /** \struct STRXEVALSITE
                                          \brief Microdosimetric site.
                                      */ 
struct STRXEVALSITE {
   float dcos, dsin, dzz, drr;        /**< site coordinate info*/
};
                                      /** \struct STRXEVALMDOSE
                                          \brief For evaluation of microdosimetry.
                                      */ 
struct STRXEVALMDOSE {
#define TRXEVAL_PHD 0x00010000        /* request puise height distr. */                                        
   unsigned int uiFlag;               /**< flag bits */
   unsigned int uirScale;             /**< lin/log/... */
   int nr;                            /**< # of radial bins */
   int nz;                            /**< # of axial bins */
   double dvLimr[2];                  /**< radial limits [cm] */
   double dvLimz[2];                  /**< axial limits [cm] */
   double dDiameter, dR, dR2;         /**< dimensions of microdosimetric site */
   double dESum;                      /**< Dose [Gy] */
   unsigned int uiShape;              /**< sphere/cyl. */
   unsigned int uiSite;               /**< total # of sites */
   unsigned int uiThreshold;              /**< threshold # of ionizations */
   unsigned int *puiNormR;            /**< [nr] # of sites with same distance */ 
   unsigned int *puiBreaksR;          /**< [nr] # of "damages" at distance */ 
   unsigned int *pnPhi;               /**< [nr] # of azimuthal positions at distance */ 
   unsigned int ****puiHit;           /**< [nevt][iz][ir][iphi] hit distribution (ionizations) */
   double  ****pdEHit;                /**< [nevt][iz][ir][iphi] hit distribution (energy) */
   struct STRXEVALSITE ***psSite;     /**< [nz][rr][iphi] site coordinates */
   double *pdrBin;                    /**< radial bins [cm] */
   double *pdzBin;                    /**< axial bins [cm] */
};
                                      /** \struct STRXEVALOUT
                                          \brief For "outside" spectra.
                                      */ 
struct STRXEVALOUT {
   unsigned int uiEScale;             /**< lin/log/... */
   int nE, nA, nPhi;                  /**< # of energy, angular bins */                  
   double dvLimE[2];                  /**< energy limits [keV] */
   double dvLimA[2];                  /**< polar angular limits [deg] */
   double dvLimPhi[2];                /**< azimuthal angular limits [0..360 deg] */
   double dvLimPhi180[2];             /**< azimuthal angular limits [-180..180 deg] */
   double *pdEBin;                    /**< energy bins [keV] */
   double *pdABin;                    /**< angular bins (cos()) */
   unsigned int **puiSpcAE;           /**< [nA][nE] spectrum */
};
                                      /** \struct STRXEVALRDOSE
                                          \brief For evaluation of radial dose.
                                      */ 
struct STRXEVALRDOSE {
   unsigned int uirScale;            /**< lin/log/... */
   double dvLimr[2];                  /**< radial bins */
   double dvLimz[2];                  /**< axial bins */
   unsigned int uiCoreI;             /**< # of ionizations < rmin */
   unsigned int uiCoreE;             /**< # of interactions < rmin */            
   double dCoreE;                     /**< energy deposited < rmin */
   int nr;                            /**< # of radial bins */
   double *pdrBin;                    /**< radial bins [cm] */
   double *pdRDoseE;                  /**< radial dose distribution */
   unsigned int *puiRDoseI;          /**< radial ionization distribution */
};
                                      /** \struct STRXEVALRZDOSE
                                          \brief For evaluation of radial-axial dose.
                                      */ 
struct STRXEVALRZDOSE {
   unsigned int uirScale;            /**< lin/log/... */
   unsigned int uizScale;            /**< lin/log/... */
   int nr;                            /**< # of radial bins */
   int nz;                            /**< # of axial bins */
   double *pdrBin;                    /**< radial bins [cm] */
   double *pdzBin;                    /**< axial bins [cm] */
   double dvLimr[2];                  /**< radial limits [cm] */
   double dvLimz[2];                  /**< axial limits [cm] */
   char *pcrGrid;                     /**< list of radial bins [cm] */
   char *pczGrid;                     /**< list of axial bins [cm] */
   double **pdrzDoseE;                /**< [nr][nz] 2-dim energy dose [Gy] */
   unsigned int **puirzDoseI;        /**< [nr][nz] 2-dim ionization events */
};
                                      /** \struct STRXEVALSPHEREDOSE
                                          \brief For evaluation of spherical dose.
                                      */ 
struct STRXEVALSPHEREDOSE {
   unsigned int uirScale;            /**< lin/log/... */
   int nr;                            /**< # of radial bins */
   unsigned int uiCoreI;             /* # of ionizations < rmin */
   unsigned int uiCoreE;             /* # of interactions < rmin */            
   double dCoreE;
   double dvLimr[2];                  /**< radial limits [cm] */
   double dvCentre[3];                /**< sphere centre coord.s [cm] */				         
   double *pdrBin;                    /**< radial bins [cm] */
   double *pdRDoseE;            /**<  */
   unsigned int *puiRDoseI;
   unsigned int *puiRCountE;
   char *pcrGrid;                     /**< list of radial bins [cm] */
};
                                      /** \struct STRXEVALTESPC
                                          \brief For evaluation of time-energy distribution.
                                      */ 
struct STRXEVALTESPC {
   unsigned int uitScale;            /**< lin/log/... */
   unsigned int uiEScale;            /**< lin/log/... */
   int nE;                            /**< # of energy bins */                          
   int nt;                            /**< # of time bins */                         
   double dvLimt[2];                  /**< time limits [sec] */
   double dvLimE[2];                  /**< energy limits [keV] */
   double *pdtBin;                    /**< [nt+1] time bins [sec]*/
   double *pdEBin;                    /**< [nE+1] energy bins [keV] */
   unsigned int **puiSpctE;          /**< [nt][nE] spectrum */
};
                                      /** \struct STRXEVALVOLESPC
                                          \brief For evaluation of volume-energy distribution.
                                      */ 
struct STRXEVALVOLESPC {
   unsigned int uiEScale;            /**< lin/log/... */
   int nE;                            /**< # of energy bins */                                  
   int nV;                            /**< # of volumes */                                 
   double dvLimE[2];                  /**< energy limits [keV] */
   double *pdEBin;                    /**< [nE+1] energy bins [keV] */
   unsigned int **puiSpcVE;          /* [nV][nE] spectrum */
};
                                      /** \struct STRXEVALCROSSERSVOLVOL
                                          \brief crossers evaluation volume.
                                      */ 
struct STRXEVALCROSSERSVOLVOL {
   unsigned int uiID;                /**<  */
   unsigned int uiVolStart;          /**<  */
   unsigned int uiFromI;             /**<  */
   unsigned int uiFromO;             /**<  */
   double ddEFromI;                   /**<  */
   double ddEFromO;                   /**<  */
};
                                      /** \struct STRXEVALCROSSERSVOL
                                          \brief crossers evaluation volume.
                                      */ 
struct STRXEVALCROSSERSVOL {
   unsigned int uiInsiders;          /**<  */
   unsigned int uiStoppers;          /**<  */
   unsigned int uiStarters;          /**<  */
   unsigned int uiCrossers;          /**<  */
   double ddEInsiders;                /**<  */
   double ddEStoppers;                /**<  */
   double ddEStarters;                /**<  */
   double ddECrossers;                /**<  */
   
   int nVV;                           /**< # of cross volumes */  
   struct STRXEVALCROSSERSVOLVOL *psVV; /**< [nVV] */
};
                                      /** \struct STRXEVALCROSSERS
                                          \brief for evaluation of crossers et al.
                                      */ 
struct STRXEVALCROSSERS {
#define TRXEVAL_ECINC (1024)          /* realloc() increment */
   int mV;                            /**< # of volumes in evaluation */
   struct STRXEVALCROSSERSVOL *psV;   /**< [mV+1] volume descriptions */
};   
                                      /** \struct STRXEVALPOINTTRACK
                                          \brief point track structure.
                                      */ 
struct STRXEVALPOINTTRACK {
#define TRXEVAL_XSMAX (TRXXS_MAXPSEUDOXS+1)
   FILE *fpTrack[ TRXPROJ_MAX ][ TRXEVAL_XSMAX ];
   char   cTrack[ TRXPROJ_MAX ][ TRXEVAL_XSMAX ][L_tmpnam+1];
   int8_t bTrack[ TRXPROJ_MAX ][ TRXEVAL_XSMAX ];
};   
                                      /** \struct STRXEVAL
                                          \brief Evaluation main structure.
                                      */ 
struct STRXEVAL {
   unsigned int uiFlag;              /**< flag bits */
   unsigned int uiEvt;               /**< actual # of events */				      
   double dDensity;                   /**< density [g/cmm**3] */
   char *pcOutFile;                   /**< outfile */
   char *pcUsrStr;                    /**< user string */
   void *pvEval;                      /**< "private" evaluation data */
                                      /**< evaluation function */
   int (*eEval)(struct STRXEVAL *, void *, int, unsigned int, unsigned int);                       
   void *hEval;                       /**< handle for dynamically loaded evaluation */
                                      /* */
   struct SCMD *psCmd;                /**< for reference to command subsystem */
                                      /* */
   struct STRXGEO *psGeo;             /**< for reference to geometry subsystem */
                                      /* */
   struct STRXEVALADOSE sADose;
   struct STRXEVALMDOSE sMDose;
   struct STRXEVALOUT   sOut;
   struct STRXEVALRDOSE sRDose;
   struct STRXEVALRZDOSE sRZDose;
   struct STRXEVALSPHEREDOSE sSphereDose;
   struct STRXEVALTESPC sTESpc;
   struct STRXEVALVOLESPC sVolESpc;
   struct STRXEVALCROSSERS sCrossers;
   struct STRXEVALLINETRACK sLTrack;
   struct STRXEVALPOINTTRACK sPTrack;
   struct SCMDERR sCE;
};
                                      /* */
                                      /* evaluation actions */ 
enum { TRXEVAL_SETUP, TRXEVAL_EVENT, TRXEVAL_EOF, TRXEVAL_ERROR, TRXEVAL_CLEANUP, TRXEVAL_INIT, TRXEVAL_INSPECT,};
                                      /* scale */ 
enum { TRXEVAL_LIN=0, TRXEVAL_LOG };
                                      /* shape for microdosim. */ 
enum { TRXEVAL_SPHERE=0, TRXEVAL_CYL };

/*

   General
   -------

   o check proper cleanup of Eval*()-routines


   TRXEvalFree   : ( eval, flag )
                     release resources
                 3 o todo
		   o to complete:
		     o cleanup of special entries 
                   o calls
                     o TRXEvalInit()
		     
   TRXEvalInit   : 
                 2 x to test
                   o calls

   TRXEvalInspect: 
                 1 x to test
                   o calls


*/
                                      /* function prototypes */
int TRXEvalFree   ( struct STRXEVAL *, unsigned int );
int TRXEvalInit   ( struct STRXEVAL *, unsigned int );
int TRXEvalInspect( struct STRXEVAL *, unsigned int );

struct STRXEVAL *_TRXEvalTest( unsigned int );

/*
   TRXEvalPointTrack: 
                    5 x to test
                      o to complete:
                        o OS/2 branch
                      o calls

   TRXEvalLineTrack : 
                    6 o to test (formally)
		      o to complete
		        o description of energies
		        x start point not included
			x colours
			x e- only
			? interactions by symbols extra
                      o calls

   TRXEvalOPAC      :
                        evaluates OPAC stuff 
                      o to do
                      o calls

   TRXEvalOut       : 
                        evaluates energy/angle distribution on exit
                    4 x to test
                      o calls

   TRXEvalRDose     :
                        evaluates radial dose distribution 
                    1 x to test
                      o calls

   TRXEvalADose     : 
                        evaluates axial dose distribution 
                    2 x to test
                      o calls

   TRXEvalMDose     : 
                        evaluates microdosimetric quantities
                    3 x to test
		      o to complete
		        x PHD, <z> etc
			x cleanup        
		        o cylinder
			x log(r) for outer, lin(r) for inner
                      o calls

*/

int TRXEvalOut   ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
int TRXEvalTESpc ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
int TRXEvalRDose ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
int TRXEvalADose ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
int TRXEvalMDose ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
int TRXEvalOPAC  ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
int TRXEvalrzDose( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
int TRXEvalSphereDose( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
/*
int TRXEvalUsr  ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
*/
int TRXEvalCrossers  ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
int TRXEvalPointTrack( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );
int TRXEvalLineTrack ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned int, unsigned int );

int TRXEvalVolESpc( struct STRXEVAL *, struct STRXVOLEVT *, int, unsigned int, unsigned int );

                                      /* error handling */
int TRXEvalError( struct SCMDERR *, unsigned int );

                                      /* command processors */
int TRXCmdLineTrack( const char *, const char *, 
                     double *, double *, 
                     int32_t * );

int TRXCmdPointTrack( const char *, const char *, int32_t * );

int TRXCmdEvalOut  ( const char *, const char *,
                     double *, double *, int32_t *,
                     double *, double *, int32_t *,
                     double *, double *, int32_t *,
                     int32_t *, int32_t *,
                     int32_t * );

int TRXCmdEvalTESpc( const char *, const char *,
                     double *, double *, int32_t *,
                     int32_t *, int32_t *,
                     double *, double *, int32_t *,
                     int32_t *, int32_t *,
                     int32_t * );

int TRXCmdEvalRDose( const char *, const char *, 
                     double *,
                     double *, double *, 
                     double *, double *, int32_t *,
                     int32_t *, int32_t *,
                     int32_t * );

int TRXCmdEvalADose( const char *, const char *, 
                     double *, double *, 
                     double *, double *, int32_t *,
                     int32_t * );

int TRXCmdEvalOPAC ( const char *, const char *, 
                     double *,
                     double *, double *, 
                     double *, double *, int32_t *,
                     int32_t *, int32_t *,
                     int32_t * );

int TRXCmdEvalrzDose( const char *, const char *, 
                      double *,
                      double *, double *, int32_t *, int32_t *, int32_t *, const char *,
                      double *, double *, int32_t *, int32_t *, int32_t *, const char *,
                      int32_t * );

int TRXCmdEvalSphereDose( const char *, const char *, 
                          double *,
                          const char *, 
                          double *, double *, int32_t *,
                          int32_t *, int32_t *,
                          const char *, 
                          int32_t * );

int TRXCmdEvalMDose( const char *, const char *, 
                     double *, double *, 
                     double *, double *, int32_t *,
                     int32_t *, int32_t *,
                     double *, const char *,
                     int32_t *,
                     int32_t *,
		                   int32_t * );

int TRXCmdEvalCrossers( const char *, const char *,
                        const char *,   
		        int32_t * );

int TRXCmdEvalUsr  ( const char *, const char *, const char *, const char *,
                     int32_t * );

int TRXCmdEvalVolESpc( const char *, const char *,
                        double *, double *, int32_t *,
                        int32_t *, int32_t *,
		        int32_t * );

int TRXCmdEvalTest    ( const char *, int32_t * );

enum { 
   ERRTRXEVAL_SYS=1,
   ERRTRXEVAL_BADADDR,
   ERRTRXEVAL_BADLIMR,
   ERRTRXEVAL_BADFILENAME,
   ERRTRXEVAL_SHELLEXIT,
   ERRTRXEVAL_BADNEVT,
   ERRTRXEVAL_BADLIMZ,
   ERRTRXEVAL_BADLIME,
   ERRTRXEVAL_BADLIMA,
   ERRTRXEVAL_BADLIMPHI,
   ERRTRXEVAL_BADLIMT,
   ERRTRXEVAL_BADGRIDR,
   ERRTRXEVAL_BADGRIDZ,
   ERRTRXEVAL_BADNR,
   ERRTRXEVAL_BADNZ,
   ERRTRXEVAL_LOADFAILED,
   ERRTRXEVAL_BINDFAILED
};

#endif
