#ifndef TRXSRC_H
#define TRXSRC_H
/** \file
   Definitions for particle sources.
*/
/*

   Copyright  1992-2011 M.Kraemer, GSI Darmstadt 

   $Log: trxsrc.h,v $
   Revision 1.5  2009/05/29 12:37:24  kraemer
   reworked projectiles

   Revision 1.4  2008/06/20 14:29:36  kraemer
   tilted vol's

   Revision 1.3  2008/02/05 10:55:34  kraemer
   doxygenized

   Revision 1.2  2002/11/25 16:36:30  kraemer
   src2file, space

   Revision 1.1  2001/01/09 14:54:40  kraemer
   *** empty log message ***


*/

#include "cmderr.h"
#include "trxproj.h"
#include "trxrnd.h"
#include "trxvol.h"

/*
   --- Common status and action flags ----------------------------------------
*/
                                      /* */
                                      /* masks */
#define TRXSRC_MASKGLOBAL 0xFF000000  /* global (e.g. debug) */
#define TRXSRC_MASKFUNC   0x00FF0000  /* function specific flags */
#define TRXSRC_MASKQUAL   0x0000FFFF  /* for additional qualifiers */

                                      /* flags */
#define TRXSRC_DEBUG      0x80000000  /* general check flag */
#define TRXSRC_CHECK      TRXSRC_DEBUG/* general check flag */
#define TRXSRC_QUIET      0x10000000  /* suppress error messages */
#define TRXSRC_NOMSG      TRXSRC_QUIET

#define NEWSRC
#if defined(NEWSRC)
                                      /** \struct STRXSRCPROJ
                                          \brief Describes a source's projectile(s).
                                      */ 
struct STRXSRCPROJ {
   double dEnergy;                    /**< start energy (mono-energetic) */
   double dProb;                      /**< this projectile's probability */
   double dCum;                       /**< cumulative probability */
   int nE;                            /**< # energy bins */
   double *pdEBin;                    /**< [nE+1] energy bin limits */   
   double *pdECum;                    /**< [nE+1] energy spectrum cumulated */   
   double *pdESpc;                    /**< [nE]   energy spectrum */
   struct STRXPROJ sProj;             /**< projectile */ 
};
                                      /** \struct STRXSRC
                                          \brief Describes a source.
                                      */ 
struct STRXSRC {
   unsigned int uiFlag;               /**< status flag */
   size_t ulSpace;                    /**< occupied space (bytes) */
   int nSP, mSP;                      /**< current,max # of projectiles */
   struct STRXSRCPROJ *psSP;          /**< [nSP] */
   struct STRXVOL sV;                 /**< holds source geometry */
   struct STRXVOL sVDir;              /**< holds direction geometry */
};
#else
                                      /** \struct STRXSRC
                                          \brief Describes a source.
                                      */ 
struct STRXSRC {
   unsigned int uiFlag;               /**< status flag */
   size_t ulSpace;                    /**< occupied space (bytes) */
   double dEnergy;                    /**< start energy (mono-energetic) */
   int nE;                            /**< # energy bins */
   double *pdEBin;                    /**< [nE+1] energy bin limits */   
   double *pdECum;                    /**< [nE+1] energy spectrum cumulated */   
   double *pdESpc;                    /**< [nE]   energy spectrum */
   struct STRXVOL sV;                 /**< holds source geometry */
   struct STRXVOL sVDir;              /**< holds direction geometry */
   struct STRXPROJ sProj;             /**< projectile */ 
};
#endif
                                      /* */
#define TRXSRC_REFCOPY 0x000000001
                                      /* */
#define TRXSRC_FILEVERSION 20000118
#define TRXSRC_FILETYPE    "trxsrc"

/*

   TRXSrcComplete      ( src, projbase, flag ) 
                       o complete source description
                     7 x to test
                       o calls
                         o TRXVolComplete()
                         o TRXProj*()
                
   TRXSrcFree          ( src, flag ) 
                       o release resources
                     4 x to test
                       o calls
                         o TRXSrcInit()
                         o TRXVolFree()
                         o TRXProjFree()
                
   TRXSrcImportCentre  ( src, str, flag ) 
                       o import centre coordinates
                     5 x to test
                       o calls
                
   TRXSrcImportDir     ( src, str, flag ) 
                       o import directions
                     6 x to test          
                       o calls
                
   TRXSrcInit          ( src, flag )
                       o initialize 
                     2 x to test
                       o calls
                         o TRXVolInit()
                         o TRXProjInit()
                
   TRXSrcInspect       ( src, flag ) 
                       o inspect
                     1 x to test
                       o calls
                         o TRXVolInspect()
                         o TRXProjInspect()
                
   TRXSrcIsValid       ( src, flag )
                       o check valid source 
                     3 x to test
                       o calls
                         o TRXProjIsvalid()
                
   TRXSrcList          ( src, flag )
                       o list some source properties 
                     8 x to test
                       o to complete
                         o source shape output  
                           (needs support by TRXVol() )
                       o calls
                
   TRXSrcRead          ( src, file, flag )
                       o read a source description file 
                     9 x to test
                       o to complete
                         o implement direction distribution
                       o calls
                         o TRXSrcInit()
                         o TRXSrcImportDir()
                         o TRXSrcFree()
                         o TRXProj*()

   TRXSrcSpace         ( src, flag ) 
                       o estimate space occupied
                    10 x to test
                       o exact numbers to test
                       o calls
                         o TRXVolSpace()
                         o TRXProjSpace()
                
   TRXSrcWrite         ( src, prefix, list, flag ) 
                       o write to file
                    12 o to do
                       o calls
                         o TRXSrc2File()
                
   TRXSrc2File         ( src, fp, flag ) 
                       o write to file stream
                    11 o to do
                       o calls
                         o 
                

*/
                                      /* function prototypes */
struct SCMDERR TRXSrcComplete( struct STRXSRC *, struct STRXPROJBASE *, unsigned int );
struct SCMDERR TRXSrcCopy    ( struct STRXSRC *, struct STRXSRC *, unsigned int );
struct SCMDERR TRXSrcFree    ( struct STRXSRC *, unsigned int );
struct SCMDERR TRXSrcImportCentre( struct STRXSRC *, const char *, unsigned int );
struct SCMDERR TRXSrcImportDir   ( struct STRXSRC *, const char *, unsigned int );
struct SCMDERR TRXSrcInit    ( struct STRXSRC *, unsigned int );
struct SCMDERR TRXSrcInspect ( struct STRXSRC *, unsigned int );
struct SCMDERR TRXSrcIsValid ( struct STRXSRC *, unsigned int );
struct SCMDERR TRXSrcList    ( struct STRXSRC *, unsigned int );
struct SCMDERR TRXSrcSpace   ( struct STRXSRC *, unsigned int );
struct SCMDERR TRXSrcRead    ( struct STRXSRC *, const char *, unsigned int );
struct SCMDERR TRXSrcWrite   ( struct STRXSRC *, const char *, unsigned int );

struct STRXSRC _TRXSrcTest( unsigned int, unsigned int );
                                      /* error handling */
int TRXSrcError( struct SCMDERR *, unsigned int );

enum { ERRTRXSRC_SYS=1,
       ERRTRXSRC_BADADDR,
       ERRTRXSRC_BADNUMBER,
       ERRTRXSRC_BADDIRVAL,
       ERRTRXSRC_BADPROJ,
       ERRTRXSRC_BADFILEVERS,
       ERRTRXSRC_BADFILETYPE,
       ERRTRXSRC_BADENERGY,
       ERRTRXSRC_MULTIENERGY,
       ERRTRXSRC_MULTIDIR,
       ERRTRXSRC_READUNKNOWNTAG,
       ERRTRXSRC_READBADSYNTAX,
       ERRTRXSRC_READNOPROJ,
       ERRTRXSRC_READBADPROB,
       ERRTRXSRC_BADEDIST,
       ERRTRXSRC_BADDDIST,
       ERRTRXSRC_SHAPENOTUNIQUE,
       ERRTRXSRC_BADSHAPE,
       ERRTRXSRC_BADHEIGHT,
       ERRTRXSRC_BADRADIUS,
       ERRTRXSRC_BADANGLE,
       ERRTRXSRC_BADSIZE,
       ERRTRXSRC_BADTHICKNESS,
       ERRTRXSRC_NOSTARTENERGY,
       ERRTRXSRC_DUPLICATEPROJ,
};

#endif
