#ifndef UFILEMEM_H
#define UFILEMEM_H
/** \file Definitions for file/memory interface.
*/
/*
   Copyright  2002-2009 M.Kraemer, D-64291 Darmstadt, Seegartenstr. 32

   $Log: ufilemem.h,v $
   Revision 1.1.1.1  2009/05/06 16:21:41  kraemer


   Revision 1.1.2.3  2009/05/06 14:03:19  kraemer
   64bit-clean

   Revision 1.1.2.2  2005/03/11 18:31:53  kraemer
   MacOS X sync

   Revision 1.1.2.1  2005/01/31 20:40:36  kraemer
   *** empty log message ***


*/

#include <sys/types.h>
#include <sys/stat.h>

#if defined(AMIGA)
 
#include <dos.h>
#include <clib/dos_protos.h>
#include <proto/exec.h>
#include <exec/memory.h>
 
#elif defined(__EMX__)

#else

#include <sys/mman.h>
#include <fcntl.h>

#endif

#include "ummap.h"

                                      /* */
#define UFILEMEM_ERRORMASK 0xFF000000
#define UFILEMEM_ERROR     0x80000000
#define UFILEMEM_WARNING   0x40000000

#define UFILEMEM_MASKFUNC   0x00FF0000/* function specific flags */

#define UFILEMEM_DEBUG  0x80000000    /* general check flag */
#define UFILEMEM_QUIET  0x10000000    /* suppress error messages */
#define UFILEMEM_NOMSG  UFILEMEM_QUIET
                                      /*  */ 
                                      /** \struct SFILEMEM
                                          \brief Describes file-memory mapping.
                                      */ 
struct SFILEMEM {
   unsigned int uiFlag;               /**< flag bits (see below) */
#define UFILEMEM_PVALLOC    0x00000001  /* pv allocated, not mmap()ed */
#define UFILEMEM_LINESTRDUP 0x00000002  /* line contents by strdup() */
   off_t offFileSize;                 /**< filesize (32/64 bit) in bytes */
   unsigned int uiFileType;           /**< plain/zip/... */ 
   FILE *fp;                          /**< stream for (un)compress */
   void *pv; off_t offv;              /**< uncompressed data: addr, length
                                           (might be reference to mmap)  
                                      */   
#undef UFILEMEMWITHLINES 
#if defined(UFILEMEMWITHLINES)
   char **pvcLines;                   /**< text files: line pointers */
   size_t *nvcLines;                  /**< text files: line length */
#else
   char *pcNext;                      /**< text files: pointer to next line */
   char *pcLine;                      /**< text files: pointer to current line */
   off_t ncLine;                      /**< text files: current line length */
   off_t offFile;                     /**< text files: remaining file length */
#endif
   struct SUMMAP sMM;
};
                                      /* file types */
enum { 
   UFILEMEM_PLAIN=0, UFILEMEM_COMPRESS, UFILEMEM_GZIP, UFILEMEM_ZIP, UFILEMEM_LHA, 
};
                                      /* mode flags */
#define UFILEMEM_READ   0x00000001    /* file open for read */
#define UFILEMEM_WRITE  0x00000002    /* file open for write */
#define UFILEMEM_APPEND 0x00000004    /* file open for append */
#define UFILEMEM_STREAM 0x00000010    /* file open for stream output */

                                      /* function action flags */
#define UFILEMEMCLOSE_FILEONLY 0x00010000                                       
/*

  uFileMemClose   ( fm, flag ) 
                  o close files, release related resources
                3 o to test
                    x A
                    x O
                    . X
                  o calls
                    o uMMapClose() 

  uFileMemCopy    ( fmto, fmfr, flag )
                  o plain copy, no duplicate, account for linked lists 
                5 o to test
                    x A
                    x O
                    . X
                  o calls

  uFileMemFree    ( fm, flag ) 
                4 o to test
                    x A 
                    x O
                    . X
                  o calls
                    o uFileMemInit()
                    o uFileMemClose()
                    o uMMapFree()

  uFileMemInit    ( fm, flag ) 
                2 o to test
                    x A
                    x O
                    o X
                  o calls
                    o uMMapInit() 

  uFileMemInspect ( fm, flag ) 
                1 o to test
                    x A
                    x O
                    . X
                  o calls
                    o uMMapInspect() 

  uFileMemLines   ( fm, flag )
                  o for text files: establish line arrays 
                8 o obsolete
                    o A
                    x O
                    o X
                  x support 0x1A (CTRL-Z ?)
                  x inplace compress of extra \r, 0x1A
                  o calls

  uFileMemLineFirst( fm, flag )
                  o for text files: position at first line
                    (modifies entries in fm) 
               8b o to do
                    o A
                    o O
                    . X
                  o calls

  uFileMemLineNext( fm, flag )
                  o for text files: find next line
                    (modifies entries in fm) 
               8a o to test
                    o A
                    o O
                    . X
                  x support 0x1A (CTRL-Z ?)
                  x inplace compress of extra \r, 0x1A
                  o calls

  uFileMemOpen    ( fm, filename, mode, flag )
                  o open a file, prepare for read
                6 o to test
                    o A
                    x O
                    . X
                  ! no => use given filename over internal one
                  o mode: read, write, append
                  x remember compression mode
                  o all compressions on all platforms
                       .lha .Z .gz .zip
                    A:  
                    X:   
                    O:      +  +   +     
                    o get zip, compress, gzip, unzip, uncompress, gunzip
                    o executables + API
                  o calls
                    o uMMapOpen()
                    o uMMapClose()
                    o uFileMemClose()

  uFileMemRead    ( fm, flag )
                  o "read" (uncompress) file, sets fm->pv  
                7 o to test
                    x A
                    x O
                    . X  
                  o fread(): efficient buffer size ?
                  o calls

  uFileMemWrite   ( fm, flag )
                  o write data from given fm->pv or lines arrays
                9 o to test
                    o A 
                    x O 
                    . X
                  o keep fp valid    
                  o calls
*/
                                      /* function prototypes */
int uFileMemClose    ( struct SFILEMEM *, unsigned int );
int uFileMemCopy     ( struct SFILEMEM *, struct SFILEMEM *, unsigned int );
int uFileMemFree     ( struct SFILEMEM *, unsigned int );
int uFileMemInit     ( struct SFILEMEM *, unsigned int );
int uFileMemInspect  ( struct SFILEMEM *, unsigned int );
#if defined(UFILEMEMWITHLINES)
int uFileMemLines    ( struct SFILEMEM *, unsigned int );
#endif
int uFileMemLineFirst( struct SFILEMEM *, unsigned int );
int uFileMemLineNext ( struct SFILEMEM *, unsigned int );
int uFileMemOpen     ( struct SFILEMEM *, const char *, unsigned int, unsigned int );
int uFileMemRead     ( struct SFILEMEM *, unsigned int );
int uFileMemWrite    ( struct SFILEMEM *, unsigned int );

struct SFILEMEM *_uFileMemTest( unsigned int );

int   umem2txt( void *, const char *);
void *utxt2mem( const char * );

char *ufgetln( char *, size_t *, FILE * );
#if defined(__APPLE_CC__)
#else
#define fgetln ufgetln 
#endif

char *uexpfn( const char * );

/*
   --- error handling
*/
int uFileMemStrError( int, char *, size_t );


enum { ERRUFILEMEM_SYS=1, 
       ERRUFILEMEM_BADADDR,
       ERRUFILEMEM_EOF,
       ERRUFILEMEM_NOBUFFER,
       ERRUFILEMEM_BADFILENAME,
       ERRUFILEMEM_BADFILESIZE,
       ERRUFILEMEM_STATFAILED, 
       ERRUFILEMEM_POPENFAILED, 
       ERRUFILEMEM_FOPENFAILED, 
       ERRUFILEMEM_UMMAPFAILED, 
       ERRUFILEMEM_NOMAP,
    ERRUFILEMEM_LARGEFILE,
};

#endif
